/* 
  * Country Code information For TL-WA3500RE V1 and RE210 V1
  * Rugemeng  20131203
  *
  */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>

#include "common.h"
#include "countryCode.h"
#include "CountryName.h"
#include "country_region.h"
#include "ralink_format.h"
#include "local.h"
#include "ralink_descp_format.h"
#include "rt_channel.h"

void FormName(char *name, char *nameL)
{
	int index;
	int key = 1;
	int len;

	len = strlen(name);

	for(index=0; index<len; index++)
	{
		if (key==1)
		{
			nameL[index] = toupper(name[index]);
			key = 0;
		}
		else
			nameL[index] = tolower(name[index]);

		if(name[index] == ' ')
			key = 1;
	}
}

/*
 * Form the Country Info from file "CountryName.h"
 * We get the Country'name, isoname, countrycode
 */
 
int formCountryNameInfo(CountryInfo *country, int countryNum)
{
	int len = sizeof(ahCmnAllCountries)/sizeof(COUNTRY_CODE_TO_ENUM_RD);
	int	i, j;
	int 	tmp;
	
	for (j=0; j<countryNum; j++)
	{
		for (i=0; i<len; i++)
		{
			if( country[j].countryCode == ahCmnAllCountries[i].countryCode)
			{
				tmp = strlen(ahCmnAllCountries[i].isoName);
				strncpy(country[j].isoName, ahCmnAllCountries[i].isoName, tmp );
				country[j].isoName[tmp] = '\0';

				tmp = strlen(ahCmnAllCountries[i].name);
				strncpy(country[j].name, ahCmnAllCountries[i].name, tmp);
				country[j].name[tmp] = '\0';

				FormName(country[j].name, country[j].nameL);

				break;
			}
		}
		if (i>=len)
			printf("Can't find Country name to the Country Code:%d\n", country[j].countryCode);
	}
	return 1;
}

/*
 * Form country's region info from "country_region.h"
 * For country UNISTED STATES, CANADA, TAIWAN, BRAZIL that they use FCC
 */
int formRegion(COUNTRY_AUTH countryAuth)
{
	if( countryAuth.countryCode == CTRY_UNITED_STATES
		|| countryAuth.countryCode == CTRY_UNITED_STATES2
		|| countryAuth.countryCode == CTRY_UNITED_STATES_FCC49
		|| countryAuth.countryCode == CTRY_CANADA
		|| countryAuth.countryCode == CTRY_TAIWAN
		|| countryAuth.countryCode == CTRY_BRAZIL )
	{
		return REGION_FCC;
	}
	if ( countryAuth.regDmnEnum >= NULL1_ETSIB && countryAuth.regDmnEnum <= NULL1_ETSIC 
		|| countryAuth.regDmnEnum == FCC2_ETSIC
		|| countryAuth.regDmnEnum == FCC3_ETSIC
		|| countryAuth.regDmnEnum == ETSI1_WORLD 
		|| countryAuth.regDmnEnum == ETSI3_ETSIA
		|| countryAuth.regDmnEnum == ETSI2_WORLD
		|| countryAuth.regDmnEnum == ETSI3_WORLD
		|| countryAuth.regDmnEnum == ETSI4_WORLD
		|| countryAuth.regDmnEnum == ETSI4_ETSIC
		|| countryAuth.regDmnEnum == ETSI5_WORLD
		|| countryAuth.regDmnEnum == ETSI6_WORLD
		|| countryAuth.regDmnEnum == ETSI8_WORLD
		|| countryAuth.regDmnEnum == ETSI9_WORLD
		|| countryAuth.regDmnEnum == ETSI_RESERVED
		|| countryAuth.regDmnEnum == APL1_ETSIC
		|| countryAuth.regDmnEnum == APL2_ETSIC
		|| countryAuth.regDmnEnum == WOR5_ETSIC
		|| countryAuth.regDmnEnum == ETSI1
		|| countryAuth.regDmnEnum == ETSI2
		|| countryAuth.regDmnEnum == ETSI3
		|| countryAuth.regDmnEnum == ETSI4
		|| countryAuth.regDmnEnum == ETSI5
		|| countryAuth.regDmnEnum == ETSI6
		|| countryAuth.regDmnEnum == ETSI8
		|| countryAuth.regDmnEnum == ETSI9
		|| countryAuth.regDmnEnum == ETSIA
		|| countryAuth.regDmnEnum == ETSIB
		|| countryAuth.regDmnEnum == ETSIC)
	{
		return REGION_CE;
	}
	if ( countryAuth.regDmnEnum != 0 )
		return REGION_OTHER;
	
	return REGION_NULL;
}

int formCountryRegionInfo(CountryInfo *country, int *countryNum)
{
	int len ;
	int index1;

	len = sizeof(allCountryAuto)/sizeof(COUNTRY_AUTH);

	if ( COUNTRY_NUM < len )
	{
		printf("The define num of country is short!\n");
		exit(0);
	}
	
	for(index1=0; index1<len; index1++)
	{
		country[index1].countryCode = allCountryAuto[index1].countryCode;
			
		country[index1].region = formRegion(allCountryAuto[index1]);
	}

	*countryNum = len;
	
	return 1;
}

void *map_file(const char *filename, unsigned long *size)
{
	int fd;
	struct stat st;
	void *map;
	
	fd = open(filename, O_RDONLY);
	if( fd < 0 || fstat(fd, &st)!=0 )
	{
		printf("Open the %s fail\n", filename);
		exit(0);
	}
	
	*size = st.st_size;
	map = mmap(NULL, *size, PROT_READ, MAP_PRIVATE, fd, 0);
	close(fd);
	
	if ( map==MAP_FAILED)
	{
		printf("Map the %s fail\n", filename);
		exit(0);
	}
	return map;
}
void release_file(void *file, unsigned long size)
{
	munmap(file, size);
}
typedef struct _SIMILAR_COUNTRY {
	CTRY_CODE	countryCode;
	struct _SIMILAR_COUNTRY *next;
}Similar;

int formCountryChannelInfo(CountryInfo *country, BAND band, int countryNum)
{
	UINT16 channelNum;
	char *file;
	
	ChannelInfo *channelInfo;
	UINT32	chBit = 0;
	unsigned long size;
	char *buf;
	char *pbuf;
	char *ptmp;
	int tmplen;
	
	Similar *countrylist = NULL;
	Similar *plist = NULL;
	int isFirst = 1;
	
	int channel, flag, extFlag, maxTxPower;
	int chNum;
	
	if (band == BAND_2G)
	{
		channelNum = CHANNEL_2G_NUM;
		file = FILE2G;
	}
	else if (band == BAND_5G)
	{
		channelNum = CHANNEL_5G_NUM;
		file = FILE5G;
	}
	
	channelInfo = (ChannelInfo *)malloc(sizeof(ChannelInfo) * channelNum);
	
	if (channelInfo == NULL)
	{
		printf("Apply mem fail\n");
		exit(0);
	}
	
	buf = (char *)map_file(file, &size);
	
	/* Skip the comment */
	pbuf = strstr(buf, "\n");
	pbuf ++;
	while( pbuf != NULL )
	{
		isFirst = 1;
		countrylist = NULL;
		while( (pbuf != NULL) && (memcmp(pbuf, "country", strlen("country")) == 0))
		{
			int countrycode;
			Similar *ptmp = NULL;
			
			tmplen = sscanf(pbuf, "country:%d\n", &countrycode);

			PRINT("Find country code: %d, return len:%d\n", countrycode, tmplen);
			
			ptmp = (Similar *)malloc(sizeof(Similar));
			ptmp->countryCode = countrycode;
			ptmp->next = NULL;
			
			if(isFirst)
			{
				isFirst = 0;			
				countrylist = ptmp;
			}
			else
			{
				plist->next = ptmp;
			}
			plist = ptmp;

			pbuf = strstr(pbuf, "\n");
			if (pbuf == NULL )
				break;
			pbuf ++;	
		}
		if (isFirst  == 1)
		{
			pbuf = strstr(pbuf, "\n");
			if (pbuf == NULL )
				break;
			pbuf ++;	
		}
		
		chNum = -1;
		chBit = 0;
		memset(channelInfo, 0, sizeof(ChannelInfo)*channelNum);
		while( (pbuf != NULL) &&  (memcmp(pbuf, "country", strlen("country")) != 0))
		{	
			chNum ++;
			if (chNum >= channelNum)
			{
				printf("The channel is overflow if file %s\n", file);
				exit(0);
			}
			 
			tmplen = sscanf(pbuf, "%d,%x,%x,%d\n", &channel, &flag, &extFlag, &maxTxPower);
			PRINT("func:%s(%d), Get the Channel %d\n",  __func__, __LINE__, channel);
			
			if ( band == BAND_2G)
				channel = INDEX_2G_CHANNEL_FREQ(channel);
			else if (band == BAND_5G)
				channel = INDEX_5G_CHANNEL_FREQ(channel);

			PRINT("func:%s(%d), Get the channel index:%d\n", __func__, __LINE__, channel);
			if (channel == -1)
			{
				PRINT("The channel is error in file %s, we skip this channel\n", file);
				chNum --;
				
				pbuf = strstr(pbuf, "\n");
				if (pbuf == NULL )
					break;
				pbuf ++;
				continue;
			}
			channelInfo[channel].support = 1;
			channelInfo[channel].flag = flag;
			channelInfo[channel].extFlag = extFlag;
			channelInfo[channel].maxTxPower = maxTxPower;
			channelInfo[channel].Geography = BOTH;
			if ( band == BAND_5G  
				&& channel > 3 && channel <19 )
				channelInfo[channel].DfsReq = YES;
			else
				channelInfo[channel].DfsReq = NO;
			chBit |= 0x01<<channel;

			pbuf = strstr(pbuf, "\n");
			if (pbuf == NULL )
				break;
			pbuf ++;
		}

		PRINT("func:%s(%d), Get the ch bit map:0x%8x\n", __func__, __LINE__, chBit);
		
		while(countrylist != NULL)
		{
			int index;
			for(index=0; index<countryNum; index++)
			{
				if(country[index].countryCode == countrylist->countryCode)
				{
					if (band == BAND_2G)
					{
						SET24G(country[index]);
						memcpy(country[index].channelInfo2G, channelInfo, sizeof(country[index].channelInfo2G));
						country[index].ch2GBit = chBit;
					}
					else if (band == BAND_5G)
					{
						SET5G(country[index]);
						memcpy(country[index].channelInfo5G, channelInfo, sizeof(country[index].channelInfo5G));
						country[index].ch5GBit = chBit;
					}
					break;
				}
			}
			if(index>=countryNum)
				printf("Can't query the country:%d\n", countrylist->countryCode);
	
			plist = countrylist->next;
			free(countrylist);
			countrylist=plist;
		}
	}
	free(channelInfo);
	release_file((void *)buf, size);
	return 1;
}

int Set5GMaxTxPower(CountryInfo *country, int countryNum)
{
	// TODO CODE
	/* 
	 * For now, we set 2G's Max TxPower is 30
	 * 5G's Max TxPower is 15
	 */
	 int index;
	int i,k;

	for ( index=0; index<countryNum; index++ )
	{
		for(i=0; i<CHANNEL_2G_NUM; i++)
		{
			if (country[index].channelInfo2G[i].support)
				country[index].channelInfo2G[i].maxTxPower = 30;
		}
	}
	for ( index=0; index<countryNum; index++ )
	{
		for(i=0; i<CHANNEL_5G_NUM; i++)
		{
			if (country[index].channelInfo5G[i].support)
				country[index].channelInfo5G[i].maxTxPower = 15;
		}
	}

	return 0;
}

extern CH_DESP Country_DB_ChDesp[];
extern CH_DESP Country_NA_ChDesp[];
extern CH_DESP Country_AL_ChDesp[];
extern CH_DESP Country_DZ_ChDesp[];
extern CH_DESP Country_AR_ChDesp[];
extern CH_DESP Country_AM_ChDesp[];
extern CH_DESP Country_AW_ChDesp[];
extern CH_DESP Country_AU_ChDesp[];
extern CH_DESP Country_AT_ChDesp[];
extern CH_DESP Country_AZ_ChDesp[];
extern CH_DESP Country_BS_ChDesp[];
extern CH_DESP Country_BH_ChDesp[];
extern CH_DESP Country_BD_ChDesp[];
extern CH_DESP Country_BB_ChDesp[];
extern CH_DESP Country_BY_ChDesp[];
extern CH_DESP Country_BE_ChDesp[];
extern CH_DESP Country_BZ_ChDesp[];
extern CH_DESP Country_BM_ChDesp[];
extern CH_DESP Country_BO_ChDesp[];
extern CH_DESP Country_BA_ChDesp[];
extern CH_DESP Country_BR_ChDesp[];
extern CH_DESP Country_BN_ChDesp[];
extern CH_DESP Country_BG_ChDesp[];
extern CH_DESP Country_KH_ChDesp[];
extern CH_DESP Country_CA_ChDesp[];
extern CH_DESP Country_CL_ChDesp[];
extern CH_DESP Country_CN_ChDesp[];
extern CH_DESP Country_CO_ChDesp[];
extern CH_DESP Country_CR_ChDesp[];
extern CH_DESP Country_HR_ChDesp[];
extern CH_DESP Country_CY_ChDesp[];
extern CH_DESP Country_CZ_ChDesp[];
extern CH_DESP Country_DK_ChDesp[];
extern CH_DESP Country_DO_ChDesp[];
extern CH_DESP Country_EC_ChDesp[];
extern CH_DESP Country_EG_ChDesp[];
extern CH_DESP Country_SV_ChDesp[];
extern CH_DESP Country_EE_ChDesp[];
extern CH_DESP Country_FI_ChDesp[];
extern CH_DESP Country_FR_ChDesp[];
extern CH_DESP Country_GE_ChDesp[];
extern CH_DESP Country_DE_ChDesp[];
extern CH_DESP Country_GR_ChDesp[];
extern CH_DESP Country_GL_ChDesp[];
extern CH_DESP Country_GD_ChDesp[];
extern CH_DESP Country_GU_ChDesp[];
extern CH_DESP Country_GT_ChDesp[];
extern CH_DESP Country_HT_ChDesp[];
extern CH_DESP Country_HN_ChDesp[];
extern CH_DESP Country_HK_ChDesp[];
extern CH_DESP Country_HU_ChDesp[];
extern CH_DESP Country_IS_ChDesp[];
extern CH_DESP Country_IN_ChDesp[];
extern CH_DESP Country_ID_ChDesp[];
extern CH_DESP Country_IR_ChDesp[];
extern CH_DESP Country_IE_ChDesp[];
extern CH_DESP Country_IL_ChDesp[];
extern CH_DESP Country_IT_ChDesp[];
extern CH_DESP Country_JM_ChDesp[];
extern CH_DESP Country_JP_ChDesp[];
extern CH_DESP Country_JO_ChDesp[];
extern CH_DESP Country_KZ_ChDesp[];
extern CH_DESP Country_KE_ChDesp[];
extern CH_DESP Country_KP_ChDesp[];
extern CH_DESP Country_KR_ChDesp[];
extern CH_DESP Country_KW_ChDesp[];
extern CH_DESP Country_LV_ChDesp[];
extern CH_DESP Country_LB_ChDesp[];
extern CH_DESP Country_LI_ChDesp[];
extern CH_DESP Country_LT_ChDesp[];
extern CH_DESP Country_LU_ChDesp[];
extern CH_DESP Country_MO_ChDesp[];
extern CH_DESP Country_MK_ChDesp[];
extern CH_DESP Country_MY_ChDesp[];
extern CH_DESP Country_MT_ChDesp[];
extern CH_DESP Country_MU_ChDesp[];
extern CH_DESP Country_MX_ChDesp[];
extern CH_DESP Country_MC_ChDesp[];
extern CH_DESP Country_MA_ChDesp[];
extern CH_DESP Country_NP_ChDesp[];
extern CH_DESP Country_NL_ChDesp[];
extern CH_DESP Country_AN_ChDesp[];
extern CH_DESP Country_NZ_ChDesp[];
extern CH_DESP Country_NI_ChDesp[];
extern CH_DESP Country_NO_ChDesp[];
extern CH_DESP Country_OM_ChDesp[];
extern CH_DESP Country_PK_ChDesp[];
extern CH_DESP Country_PA_ChDesp[];
extern CH_DESP Country_PG_ChDesp[];
extern CH_DESP Country_PY_ChDesp[];
extern CH_DESP Country_PE_ChDesp[];
extern CH_DESP Country_PH_ChDesp[];
extern CH_DESP Country_PL_ChDesp[];
extern CH_DESP Country_PT_ChDesp[];
extern CH_DESP Country_PR_ChDesp[];
extern CH_DESP Country_QA_ChDesp[];
extern CH_DESP Country_RO_ChDesp[];
extern CH_DESP Country_RU_ChDesp[];
extern CH_DESP Country_RW_ChDesp[];
extern CH_DESP Country_SA_ChDesp[];
extern CH_DESP Country_RS_ChDesp[];
extern CH_DESP Country_ME_ChDesp[];
extern CH_DESP Country_SG_ChDesp[];
extern CH_DESP Country_SK_ChDesp[];
extern CH_DESP Country_SI_ChDesp[];
extern CH_DESP Country_ZA_ChDesp[];
extern CH_DESP Country_ES_ChDesp[];
extern CH_DESP Country_LK_ChDesp[];
extern CH_DESP Country_SE_ChDesp[];
extern CH_DESP Country_CH_ChDesp[];
extern CH_DESP Country_SY_ChDesp[];
extern CH_DESP Country_TW_ChDesp[];
extern CH_DESP Country_TZ_ChDesp[];
extern CH_DESP Country_TH_ChDesp[];
extern CH_DESP Country_TT_ChDesp[];
extern CH_DESP Country_TN_ChDesp[];
extern CH_DESP Country_TR_ChDesp[];
extern CH_DESP Country_UG_ChDesp[];
extern CH_DESP Country_UA_ChDesp[];
extern CH_DESP Country_AE_ChDesp[];
extern CH_DESP Country_GB_ChDesp[];
extern CH_DESP Country_US_ChDesp[];
extern CH_DESP Country_PS_ChDesp[];
extern CH_DESP Country_UY_ChDesp[];
extern CH_DESP Country_UZ_ChDesp[];
extern CH_DESP Country_VE_ChDesp[];
extern CH_DESP Country_VN_ChDesp[];
extern CH_DESP Country_YE_ChDesp[];
extern CH_DESP Country_ZW_ChDesp[];

int formGeogDFSReqInfo(CountryInfo *country, int countryNum)
{
	int index;
	int len, i,k;
	int firstCh;
	BAND band;
	CH_DESP *tmpDesp;

	for (index=0; index<countryNum; index++)
	{
		tmpDesp = GETDESCP(country[index].isoName);
		if( tmpDesp == NULL )
		{
			printf("func:%s(%d), don't the the desp info of country %d\n", __func__, __LINE__, country[index].countryCode);
			continue;
		}
		i = 0;
		while(tmpDesp[i].FirstChannel != 0 )
		{
			firstCh = tmpDesp[i].FirstChannel;
			if ( firstCh >= 1 && firstCh <= 14 )
			{
				firstCh = INDEX_2G_CHANNEL_NUM(firstCh);
				band = BAND_2G;
			}
			else if (firstCh >= CHANNEL_5G_BAND1_NUM_START && firstCh <= CHANNEL_5G_BAND4_NUM_STOP )
			{
				firstCh = INDEX_5G_CHANNEL_NUM(firstCh);
				band = BAND_5G;
			}
			else
			{
				printf("func:%s(%d), channel:%d we don't support in country:%d\n", __func__, __LINE__, firstCh, country[index].countryCode);
				i++;
				continue;
			}
			
			len = tmpDesp[i].NumOfCh;
			for(k=firstCh; k<firstCh + len; k++)
			{
				if ( band == BAND_2G )
				{
					if ( !country[index].channelInfo2G[k].support )
					{
						printf("func:%s(%d), channel:%d we don't support in country:%d\n", __func__, __LINE__, k, country[index].countryCode);
						continue;
					}
					
					country[index].channelInfo2G[k].Geography = tmpDesp[i].Geography;
					country[index].channelInfo2G[k].DfsReq = FALSE;  /*tmpDesp[i].DfsReq; */
				}
				else if ( band == BAND_5G )
				{
					if ( !country[index].channelInfo5G[k].support )
					{
						printf("func:%s(%d), channel:%d we don't support in country:%d\n", __func__, __LINE__, k, country[index].countryCode);
						continue;
					}
				
					country[index].channelInfo5G[k].Geography = tmpDesp[i].Geography;
					country[index].channelInfo5G[k].DfsReq = tmpDesp[i].DfsReq;
				}
			}
			i++;
		}
	}
	return 0;
}

int formGetAtherosWebFlag(CountryInfo *country, int countryNum)
{
	
	return 0;
}

FILE *FormFopen(const char *fileName)
{
	char prefix[] = "File/";
	FILE *file;
	char name[100];

	if (strlen(fileName) + strlen(prefix) > 100)
	{
		printf("func:%(%d), filename is too long\n", __func__, __LINE__);
		exit(0);
	}

	sprintf(name, "%s%s", prefix, fileName);
	printf("filename is:%s\n", name);
	
	file = fopen(name, "wa+");
	if (file == NULL )
	{
		printf("Open the output file %s fail\n", name);
		exit(0);
	}
	
	fprintf(file, "/*\n");
	fprintf(file, " * Automatically generated file, don't exit\n");
	fprintf(file, " * If you wantn't fix this file, please fix the func %s is file %s\n", __func__, __FILE__);
	fprintf(file, " * Support by Rugemeng \n");
	fprintf(file, " */\n");

	return file;
}

void FormFclose(FILE *file)
{
	if ( 0 != fclose(file))
	{
		printf("Close file faile\n");
		exit(0);
	}
}

void FormPrintCountryInfo_Ralink(CountryInfo *country, int countryNum)
{
	#define OUTPUTFILE "Ralink_format_country_info.h"
	char *filename = OUTPUTFILE;
	FILE *file;
	int index;
	
	file = FormFopen(filename);
	
	fprintf(file, "COUNTRY_CODE_TO_COUNTRY_REGION allCountry[] = { \n");
	
	for(index=0; index<countryNum; index++)
	{
		char name[40];
		sprintf(name, "\"%s\"", country[index].name);
		fprintf(file, "\t{%d,\t\"%s\",\t%-30s,\t%s,\t%s,\t%s,\t%s},\n",
						country[index].countryCode,
						country[index].isoName,
						name,
						TEST5G(country[index])?"TRUE":"FALSE",
						CHKRLKABAND(country[index]),
						TEST24G(country[index])?"TRUE":"FALSE",
						CHKRLKGBAND(country[index]));
	}
	fprintf(file, "\t{999,\t\"\",\t\"\",\t0,\t0,\t0,\t0}");
	fprintf(file, "};\n");
	FormFclose(file);
	#undef OUTPUTFILE
}

void FormPrintCountryInfo_Ralink_Descp(CountryInfo *country, int countryNum)
{
	#define OUTPUTFILE "Ralink_format_Descp_Struct.h"
	char *filename = OUTPUTFILE;
	FILE *file;
	int index;
	int firstCh, firstChIndex;
	int numCh, firstTxPower, firstGeo, firstDfs;
	int print;
	int i,k;

	file = FormFopen(filename);

	for( index=0; index<countryNum; index++)
	{
		fprintf(file, "/*%s*/\n",country[index].name);
		fprintf(file, "CH_DESP Country_%s_ChDesp[] = \n{\n", country[index].isoName );

		/* 2G descp Info */
		i=0; print = 1;
		while(i<CHANNEL_2G_NUM)
		{
			if ( i==0 || (country[index].channelInfo2G[i].support && numCh == 0) )
				goto reset2G;
			if ( !country[index].channelInfo2G[i].support && numCh == 0 )
			{
				i++;
				continue;
			}
			
			if ( (!country[index].channelInfo2G[i].support  && numCh != 0 )
				|| country[index].channelInfo2G[i].maxTxPower != firstTxPower
				|| country[index].channelInfo2G[i].Geography != firstGeo
				|| country[index].channelInfo2G[i].DfsReq != firstDfs )
			{
print2G:
				fprintf(file, "\t{ %d,\t%4d,  %d, %s, %s},\n",
					firstCh,
					numCh,
					firstTxPower,
					(firstGeo == ODOR) ? "ODOR" : ( (firstGeo == IDOR) ? "IDOR" : "BOTH"),
					firstDfs ? "TRUE" : "FALSE");

reset2G:
				firstChIndex = i;
				firstCh = INDEX_2G_TO_CHANNEL(firstChIndex);
				firstTxPower = country[index].channelInfo2G[i].maxTxPower;
				firstGeo = country[index].channelInfo2G[i].Geography;
				firstDfs = country[index].channelInfo2G[i].DfsReq;
				numCh = 0;	
			}
			if( country[index].channelInfo2G[i].support )
				numCh ++;
			i++;
		}
		if ( country[index].channelInfo2G[i-1].support == 1 && print == 1)
		{
			i--;
			print = 0;
			goto print2G;
		}

		/* 5G Descp Info */
		i=0; print = 1;
		while(i<CHANNEL_5G_NUM)
		{
			if ( i==0 || (country[index].channelInfo5G[i].support && numCh == 0) )
				goto reset5G;
			if ( !country[index].channelInfo5G[i].support && numCh == 0 )
			{
				i++;
				continue;
			}
			
			if ( (!country[index].channelInfo5G[i].support  && numCh != 0 )
				|| country[index].channelInfo5G[i].maxTxPower != firstTxPower
				|| country[index].channelInfo5G[i].Geography != firstGeo
				|| country[index].channelInfo5G[i].DfsReq != firstDfs
				|| INDEX_5G_TO_CHANNEL(i) == CHANNEL_5G_BAND2_NUM_START
				|| INDEX_5G_TO_CHANNEL(i) == CHANNEL_5G_BAND3_NUM_START
				|| INDEX_5G_TO_CHANNEL(i) == CHANNEL_5G_BAND4_NUM_START )
			{
print5G:
				fprintf(file, "\t{ %d,\t%4d,  %d, %s, %s},\n",
					firstCh,
					numCh,
					firstTxPower,
					(firstGeo == ODOR) ? "ODOR" : ( (firstGeo == IDOR) ? "IDOR" : "BOTH"),
					firstDfs ? "TRUE" : "FALSE");

reset5G:
				firstChIndex = i;
				firstCh = INDEX_5G_TO_CHANNEL(firstChIndex);
				firstTxPower = country[index].channelInfo5G[i].maxTxPower;
				firstGeo = country[index].channelInfo5G[i].Geography;
				firstDfs = country[index].channelInfo5G[i].DfsReq;
				numCh = 0;	
			}
			if( country[index].channelInfo5G[i].support )
				numCh ++;
			i++;
		}
		if ( country[index].channelInfo5G[i-1].support == 1 &&  print == 1)
		{
			i--;
			print = 0;
			goto print5G;
		}
		
		fprintf(file, "\t{0}\n};\n\n\n");
	}

	/* Print Region Info */

	fprintf(file, "CH_REGION ChRegion[] = \n{\n");
	for( index=0; index<countryNum; index++)
	{
		fprintf(file, "\t{\"%s\", %s,\tCountry_%s_ChDesp},  /* %s */\n",
			country[index].isoName,
			(country[index].region == REGION_FCC) ? "FCC" : ((country[index].region == REGION_CE) ? "CE" : "OTHER"),
			country[index].isoName,
			country[index].name);
	}
	fprintf(file, "\t{\"\",0,  NULL} /* END */\n};\n");

	/* Print 2G Region Info */
	fprintf(file, "CH_REGION ChRegion2[] = \n{\n");
	for( index=0; index<countryNum; index++)
	{
		fprintf(file, "\t{\"%s\", %s},  /* %s */\n",
			country[index].isoName,
			(country[index].region == REGION_FCC) ? "FCC" : ((country[index].region == REGION_CE) ? "CE" : "OTHER"),
			country[index].name);
	}
	fprintf(file, "\t{\"\",0,  NULL} /* END */\n};\n");

	
	FormFclose(file);
	#undef OUTPUTFILE
}

void FormPrintCountryInfo_Ralink_Webserver_Javascript_Region(CountryInfo *country, int countryNum)
{
	char filename[] = "Ralink_form_webserver_javascript_region.h";
	FILE *file;
	int index;
	int abandRegion;
	
	file = FormFopen(filename);

	fprintf(file, "var regionInfo = new Array(\n");
	for (index=0; index<countryNum; index++)
	{
		fprintf(file, "\t\"id=\\\"t_%c%c%c\\\">%s<\", %d, %d, %d,\n",
			tolower(country[index].name[0]),
			tolower(country[index].name[1]),
			tolower(country[index].name[2]),
			country[index].nameL,
			1,
			country[index].ch2GBit == G_BAND_REGION_0_BIT ? 11 : (country[index].ch2GBit == G_BAND_REGION_5_BIT? 14 : 13),
			country[index].countryCode == CTRY_CHINA ? 1 : 2);
	}
	fseek(file, -2, SEEK_CUR);
	fprintf(file, "\n);\n\n\n");

	fprintf(file, "var region5GInfo = new Array(\n");
	for (index=0; index<countryNum; index++)
	{
		abandRegion = GETRLKABANDNUM(country[index]);
		fprintf(file, "\t\"id=\\\"t_%c%c%c\\\">%s<\", %d,\n",
			tolower(country[index].name[0]),
			tolower(country[index].name[1]),
			tolower(country[index].name[2]),
			country[index].nameL,
			abandRegion);
	}
	fseek(file, -2, SEEK_CUR);
	fprintf(file, "\n);\n");

	FormFclose(file);
}



void FormPrintCountryInfo_Atheros_Webserver_Javascript_Region(CountryInfo *country, int countryNum)
{
	char filename[] = "Atheros_form_webserver_javascript_region.h";
	FILE *file;
	int index, idx, tmp;
	int abandRegion;
	
	file = FormFopen(filename);

	fprintf(file, "var Region_chan_table = new Array(\n");

	for (index=0; index<countryNum; index++)
	{
		fprintf(file,"\t\"id=\\\"t_%c%c%c\\\">%s<\",",
			tolower(country[index].name[0]),
			tolower(country[index].name[1]),
			tolower(country[index].name[2]),
			country[index].nameL);

		/* Output the flag for channel 1-14 */
		for( idx=0; idx<14; idx++)
		{
			fprintf(file,"\t0x%04x,", LOCAL_TO_ATHEROS_FLAG(country[index].channelInfo2G[idx].flag, country[index].channelInfo2G[idx].extFlag));
		}

		/* Output the flag for channel 16, 20, 30, 34 */
		fprintf(file, "\t0x0,\t0x0,\t0x0,\t0x0,");

		/* Output the flag for channel 36 -52 added by 2 */
		for( idx=0; idx<5; idx++)
		{
			fprintf(file, "\t0x%04x,\t0x0,", LOCAL_TO_ATHEROS_FLAG(country[index].channelInfo5G[idx].flag, country[index].channelInfo5G[idx].extFlag));
		}

		/* Output the flag for channel 56, 60, 64 */
		tmp = idx + 3;
		for( ; idx<tmp; idx++)
		{
			fprintf(file, "\t0x%04x,", LOCAL_TO_ATHEROS_FLAG(country[index].channelInfo5G[idx].flag, country[index].channelInfo5G[idx].extFlag));
		}
		
		/* Output the flag for channel 70,80 */
		fprintf(file, "\t0x0,\t0x0,");	

		/* Output the flag for channel 100-140  */
		tmp = idx + 11;
		for( ; idx<tmp; idx++)
		{
			fprintf(file, "\t0x%04x,", LOCAL_TO_ATHEROS_FLAG(country[index].channelInfo5G[idx].flag, country[index].channelInfo5G[idx].extFlag));
		}

		/* Output the flag for channel 149-165  */
		tmp = idx + 5;
		for( ; idx<tmp; idx++)
		{
			fprintf(file, "\t0x%04x,", LOCAL_TO_ATHEROS_FLAG(country[index].channelInfo5G[idx].flag, country[index].channelInfo5G[idx].extFlag));
		}

		/* Output the flag for channel 184, 188, 192, 196  */
		fprintf(file, "\t0x0,\t0x0,\t0x0,\t0x0,");

		fprintf(file, "\n");
		
	}
	fprintf(file, "\n);\n");

	FormFclose(file);
}

CountryInfo countryInfo[COUNTRY_NUM] = {0};
int	countryNum = 0;

int main(int argc, char *argv[])
{
	/* Get the Country's region info 
 	 * For the Country Code in "Country reigion info" is the base for all, so we load it first
	 */
	formCountryRegionInfo(countryInfo, &countryNum);
	
	/* Get the Country's name, isoname, countrycode */
	formCountryNameInfo(countryInfo, countryNum);
	
	/* Get the 2G channel support */
	formCountryChannelInfo(countryInfo, BAND_2G, countryNum);
	
	/* Get the 5G channel support */
	formCountryChannelInfo(countryInfo, BAND_5G, countryNum);

	/* Set 5G's max tx power */
	Set5GMaxTxPower(countryInfo, countryNum);

	/* Get the Geography and DFS req info */
	formGeogDFSReqInfo(countryInfo, countryNum);

	/* Get the Atheros web flag */
	formGetAtherosWebFlag(countryInfo, countryNum);
	
	/* Print the country info as the Ralink format */
	FormPrintCountryInfo_Ralink(countryInfo, countryNum);

	/* Print the Descp struct info for the Ralink driver */
	FormPrintCountryInfo_Ralink_Descp(countryInfo, countryNum);

	/* Print the country info for Ralink Web javascript */
	FormPrintCountryInfo_Ralink_Webserver_Javascript_Region(countryInfo, countryNum);

	/* Print the country info for Atheros Web javascript */
	FormPrintCountryInfo_Atheros_Webserver_Javascript_Region(countryInfo, countryNum);
	
	return 0;

}



