/******************************************************************************
 *
 * Copyright (c) 2013 TP-LINK Technologies CO.,LTD.
 * All rights reserved.
 *
 * FILE NAME  :   gpio_led.c
 * VERSION    :   1.0
 * DESCRIPTION:   APIs which control LED on/off/blinking.
 *
 * AUTHOR     :   Teng Fei <tengfei@tp-link.net>
 * CREATE DATE:   04/25/2013
 *
 * HISTORY    :
 * 01   04/25/2013  Teng Fei     Create.
 *
 ******************************************************************************/
 /*
#1	INCLUDE FILES FOR MODULE
*/
#include "gpio_led.h"

/*
**	TP GPIO features
*/
#ifdef	PRODUCT_GPIO_FEATURE


/*
#3	PRIVATE FUNTIONS
*/
/*
**	config these pins to gpio mode
*/
static void gpio_setmode(void)
{
	u32	gpiomode;

	/*
	**	1. get default gpiomode
	*/
	gpiomode = le32_to_cpu(*(volatile u32 *)(RALINK_REG_GPIOMODE));


	/*
	**	2. set GPO72, GPIO40-44, GPIO0,2,3,4,13 as gpio mode.
	*/
	gpiomode |= RALINK_GPIOMODE_WLED;		/* GPO72 <---> bit13 */
	gpiomode |= RALINK_GPIOMODE_DFT;		/* GPIO1,2,7-14 <---> bit0,2-4 */

	/*
	**	GPIO24-35 <-> RGMII1_GPIO_MODE, pls refer to mt7620_datasheet.
	**	RGMII1_GPIO_MODE <-> bit9, pls refer to mt7620_ProgramingGuide.
	*/
#if defined(PRODUCT_GPIO_WA3500RE) || defined(PRODUCT_GPIO_RE210) || defined(PRODUCT_GPIO_RE210)
	gpiomode |= RALINK_GPIOMODE_GE1;		/* GPIO24-35 <---> bit9 <---> RGMII1_GPIO_MODE register */
#endif

#ifdef	PRODUCT_GPIO_WA855RE
	gpiomode |= RALINK_GPIOMODE_EPHY;		/* GPIO40-44 <---> bit15 */
	gpiomode |= RALINK_GPIOMODE_GE2;		/* GPIO60-71 <---> bit10 */
#endif	/* PRODUCT_GPIO_WA855RE */

#ifdef	PRODUCT_GPIO_WR810N
	gpiomode |= RALINK_GPIOMODE_EPHY;		/* GPIO40-44 <---> bit15 */
#endif	/* PRODUCT_GPIO_WR810N */

	/*
	**	3. write proper value to GPIOMODE register to config some pins to GPIO mode.
	*/
	*(volatile u32 *)(RALINK_REG_GPIOMODE) = cpu_to_le32(gpiomode);
}


/*
**	config dir of some pins
*/
static void gpio_setdir(void)
{
	u32 gpiodir;

	/*
	**	1. get default gpiodir of GPIO40-71, set dir of GPIO40,42 as OUT for wa855re and make it effect.
	*/
#ifdef	PRODUCT_GPIO_WA855RE
	gpiodir = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIO7140DIR));
	gpiodir |= RALINK_GPIO(RALINK_LED_RED_GPIO_NUM - 40);
	gpiodir |= RALINK_GPIO(RALINK_LED_YELLOW_GPIO_NUM - 40);
	/*
	**	write proper value to RALINK_REG_PIO7140DIR register to config directions of pin40 and pin42.
	*/
	*(volatile u32 *)(RALINK_REG_PIO7140DIR) = gpiodir;
#endif


	/*
	**	2. get default gpiodir of GPO72, set dir of GPO72 as OUT and make it effect.
	*/
	gpiodir = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIO72DIR));
#if defined(RALINK_LED_GREEN_GPIO_NUM)	
	gpiodir |= RALINK_GPIO((RALINK_LED_GREEN_GPIO_NUM -72));
#endif
#if defined(RALINK_LED_2G2_GPIO_NUM)
	gpiodir |= RALINK_GPIO((RALINK_LED_2G2_GPIO_NUM -72));	/* 2.4G wifi */
#endif
	*(volatile u32 *)(RALINK_REG_PIO72DIR) = gpiodir;

	/*
	**	3. get default gpiodir of GPIO24-39, set dir of GPIO25,27 as OUT for wa3500re/re200 and make it effect.
	*/
	gpiodir = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIO3924DIR));
#if defined(RALINK_LED_WPS_GPIO_NUM)
	gpiodir |= RALINK_GPIO((RALINK_LED_WPS_GPIO_NUM -24));	/* wps led */
#endif
#if defined(RALINK_LED_WIFI_GPIO_NUM)
	gpiodir |= RALINK_GPIO((RALINK_LED_WIFI_GPIO_NUM -24));	/* wifi led */
#endif
#if defined(RALINK_LED_SWITCH_GPIO_NUM) && (!defined(PRODUCT_GPIO_RE210))
	gpiodir |= RALINK_GPIO((RALINK_LED_SWITCH_GPIO_NUM -24)); /* Leds on/off switch */
#endif
	*(volatile u32 *)(RALINK_REG_PIO3924DIR) = gpiodir;

	/*
	**	4. get default gpiodir of GPIO0-23, set dir of GPIO2 as OUT for wa3500re/re200 and make it effect.
	*/
#ifdef	RALINK_LED_2G1_GPIO_NUM
	gpiodir = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODIR));
	gpiodir |= RALINK_GPIO((RALINK_LED_2G1_GPIO_NUM -0));	/* 2.4G wifi */
	*(volatile u32 *)(RALINK_REG_PIODIR) = gpiodir;
#endif
}



static void gpio_setpol(void)
{
	/*
	**	DO IT LATER.
	*/
}

static void gpio_led_init(void)
{
	static int	is_inited = 0;

	if (is_inited)
	{
		/* printf("gpio led has been inited!\n"); */
		return;
	}
	
	/*
	**	1. set GPIOMODE, for GPIO purpose select
	*/
	gpio_setmode();

	/*
	**	2. set proper direction of some GPIOs
	*/
	gpio_setdir();

	/*
	**	3. Now nothing to do.
	*/
	gpio_setpol();

	is_inited = 1;
}


/*
#4	EXTERNALIZED FUNCTIONS
*/
/*
**	set led on/off.
**
**	gpio:	addr of LED
**	is_on:	state --- on or off 
*/
void gpio_led_set
(
	const unsigned int gpio,
	const unsigned int is_on
)
{	
	/*
	**	1. validate gpio
	*/
#ifdef	PRODUCT_GPIO_WA855RE
	if (RALINK_LED_YELLOW_GPIO_NUM != gpio &&
		RALINK_LED_RED_GPIO_NUM != gpio &&
		RALINK_LED_GREEN_GPIO_NUM != gpio)
	{
		printf("gpio(%d) is illegal!\n", gpio);
		return;
	}
#endif
	
	/*
	**	2. gpio_led_init first.
	*/
	gpio_led_init();


	/*
	**	set LED on or off
	*/
	switch (gpio)
	{
#if defined(RALINK_LED_GREEN_GPIO_NUM) || defined(RALINK_LED_2G2_GPIO_NUM) || defined(RALINK_LED_SYS_GPIO_NUM) || defined(PRODUCT_GPIO_RE210)
#if defined(RALINK_LED_GREEN_GPIO_NUM)
	case RALINK_LED_GREEN_GPIO_NUM:
#endif	/* RALINK_LED_GREEN_GPIO_NUM */
#if defined(RALINK_LED_2G2_GPIO_NUM)
	case RALINK_LED_2G2_GPIO_NUM:
#endif	/* RALINK_LED_2G2_GPIO_NUM */
#if defined(RALINK_LED_SYS_GPIO_NUM)
	case RALINK_LED_SYS_GPIO_NUM:
#endif	/* RALINK_LED_SYS_GPIO_NUM */
#if defined(PRODUCT_GPIO_RE210)
	case RALINK_LED_2G_GPIO_NUM:
#endif	/* RALINK_LED_2G_GPIO_NUM */
		if (is_on)
		{
			/* printf("gpio led(%d) will be lighted!\n", gpio); */
			*(volatile u32 *)(RALINK_REG_PIO72RESET) |=  RALINK_GPIO((gpio-72));
		}
		else
		{
			/* printf("gpio led(%d) will be off!\n", gpio); */
			*(volatile u32 *)(RALINK_REG_PIO72SET) |= RALINK_GPIO((gpio-72));
		}
		break;
#endif	/* RALINK_LED_GREEN_GPIO_NUM or RALINK_LED_2G2_GPIO_NUM or RALINK_LED_SYS_GPIO_NUM */

#if defined(RALINK_LED_YELLOW_GPIO_NUM) || defined(RALINK_LED_RED_GPIO_NUM)
#if defined(RALINK_LED_YELLOW_GPIO_NUM)
	case RALINK_LED_YELLOW_GPIO_NUM:
#endif	/* RALINK_LED_YELLOW_GPIO_NUM */
#if defined(RALINK_LED_RED_GPIO_NUM)
	case RALINK_LED_RED_GPIO_NUM:
#endif	/* RALINK_LED_RED_GPIO_NUM */
		if (is_on)
		{
			/* printf("gpio led(%d) will be lighted!\n", gpio); */
			*(volatile u32 *)(RALINK_REG_PIO7140RESET) |=  RALINK_GPIO((gpio-40));
		}
		else
		{
			/* printf("gpio led(%d) will be off!\n", gpio); */
			*(volatile u32 *)(RALINK_REG_PIO7140SET) |=  RALINK_GPIO((gpio-40));
		}
		break;
#endif	/* RALINK_LED_YELLOW_GPIO_NUM or RALINK_LED_RED_GPIO_NUM */

#if defined(RALINK_LED_WPS_GPIO_NUM) || defined(RALINK_LED_WIFI_GPIO_NUM) || defined(RALINK_LED_SWITCH_GPIO_NUM) || defined(PRODUCT_GPIO_RE210)
#if defined(RALINK_LED_WPS_GPIO_NUM) 
	case RALINK_LED_WPS_GPIO_NUM:
#endif
#if defined(RALINK_LED_WIFI_GPIO_NUM)
	case RALINK_LED_WIFI_GPIO_NUM:
#endif     
#if defined(PRODUCT_GPIO_RE210)
    case RALINK_LED_RE1_GPIO_NUM:
    case RALINK_LED_RE2_GPIO_NUM:
    case RALINK_LED_POWER_SYSTEM_GPIO_NUM:
#endif
	case RALINK_LED_SWITCH_GPIO_NUM:
		if (is_on)
		{
			/* printf("gpio led(%d) will be lighted!\n", gpio); */
			*(volatile u32 *)(RALINK_REG_PIO3924RESET) |=  RALINK_GPIO((gpio-24));
		}
		else
		{
			/* printf("gpio led(%d) will be off!\n", gpio); */
			*(volatile u32 *)(RALINK_REG_PIO3924SET) |=  RALINK_GPIO((gpio-24));
		}
		break;
#endif	/* WPS or WIFI or SWITCH */

#if defined(RALINK_LED_2G1_GPIO_NUM) || defined(PRODUCT_GPIO_RE210)
#if defined(RALINK_LED_2G1_GPIO_NUM)
	case RALINK_LED_2G1_GPIO_NUM:
#endif 
#if defined(PRODUCT_GPIO_RE210)
	case RALINK_LED_5G_GPIO_NUM:
#endif   
		if (is_on)
		{
			/* printf("gpio led(%d) will be lighted!\n", gpio); */
			*(volatile u32 *)(RALINK_REG_PIORESET) |=  RALINK_GPIO((gpio-0));
		}
		else
		{
			/* printf("gpio led(%d) will be off!\n", gpio); */
			*(volatile u32 *)(RALINK_REG_PIOSET) |=  RALINK_GPIO((gpio-0));
		}
		break;
#endif	/* RALINK_LED_2G1_GPIO_NUM */

	default:
		/* nothing to do */
		break;
	}
}
#endif		/* PRODUCT_GPIO_FEATURE */


#ifdef FW_RECOVERY
static void gpio_fw_rcv_init(void)
{
	u32	gpiomode;
	u32 gpiodir;

	/*
	**	1. get default gpiomode and write a proper value into it.
	*/
	gpiomode = le32_to_cpu(*(volatile u32 *)(RALINK_REG_GPIOMODE));
#if (FW_BUTTON_GPIO_NUM == 2)
	gpiomode |= RALINK_GPIOMODE_I2C;		/* GPIO2 <---> bit13 */
#endif
#if (FW_LED_GPIO_NUM == 72)
	gpiomode |= RALINK_GPIOMODE_WLED;		/* GPO72 <---> bit13 */
#endif
	*(volatile u32 *)(RALINK_REG_GPIOMODE) = cpu_to_le32(gpiomode);


	/*
	**	2. get default gpiodir and write a proper value into it.
	*/
#if (FW_BUTTON_GPIO_NUM <= 23)
	gpiodir = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODIR));
	gpiodir |= RALINK_GPIO((FW_BUTTON_GPIO_NUM -0));	/* 2.4G wifi */
	*(volatile u32 *)(RALINK_REG_PIODIR) = gpiodir;
#endif
#if (FW_LED_GPIO_NUM == 72)
	gpiodir = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIO72DIR));
	gpiodir |= RALINK_GPIO((FW_LED_GPIO_NUM -72));
	*(volatile u32 *)(RALINK_REG_PIO72DIR) = gpiodir;
#endif
}


/*
**	check whether the reset button was pressed 
**	1, pressed; 0, not pressed.
*/
int gpio_indicate_fw_rcv(void)
{
	int val;
	int old_val;

	gpio_fw_rcv_init();
	udelay(10 * 1000); 		/* delay 10ms for input value stabile. by HouXB, 27Apr11 */
	old_val = le32_to_cpu(*(volatile u32 *)(FW_BTN_GPIO_ADDR));
	udelay(1000);
	val = le32_to_cpu(*(volatile u32 *)(FW_BTN_GPIO_ADDR));

	/* make sure the btn was pressed. by HouXB, 27Apr11 */
	if(old_val != val)
	{
		return 0;
	}
	val = ((val & (1 << FW_BUTTON_GPIO_NUM)) >> FW_BUTTON_GPIO_NUM);
	/* when pressed val is 0, return 1 to indicate pressed */
	return (1 - val);
}
#endif	/* FW_RECOVERY */

