/******************************************************************************
 *
 * Copyright (c) 2013 TP-LINK Technologies CO.,LTD.
 * All rights reserved.
 *
 * FILE NAME  :   ralink_gpio_product.c
 * VERSION    :   1.0
 * DESCRIPTION:   Source file to implement tp product features.
 *
 * AUTHOR     :   Teng Fei <tengfei@tp-link.net>
 * CREATE DATE:   06/19/2013
 *
 * HISTORY    :
 * 01   06/19/2013  Teng Fei     Create.
 *
 ******************************************************************************/

/*
#1	INCLUDE FILES FOR MODULE
*/
#include "ralink_gpio.h"

#if defined (PRODUCT_GPIO_FEATURE)

/*
#2	DEFINITIONS AND DECLARATIONS FOR MODULE
*/
#if defined (CONFIG_RALINK_GPIO_WA855RE)
	/*
	**	gpio number of three kinds of color LED
	*/
	/* RED indicates that system is up */
	#define RALINK_LED_RED_GPIO_NUM		42
	/* YELLOW indicates that wifi signal is weak */
	#define RALINK_LED_YELLOW_GPIO_NUM	72
	/* GREEN indicates that wifi has been connected and its signal is strong */
	#define RALINK_LED_GREEN_GPIO_NUM	40
	/*
	**	gpio number of buttons
	*/
	#define	RALINK_BTN_RST_GPIO_NUM		2
	#define	RALINK_BTN_WPS_GPIO_NUM		44
#elif defined (CONFIG_RALINK_GPIO_WR810N_EU) || defined (CONFIG_RALINK_GPIO_WR810N_US)
	/* CONFIG_RALINK_GPIO_WR810N is public Macro for wr810n series. */
	#define	CONFIG_RALINK_GPIO_WR810N
	#define	RALINK_LED_SYS_GPIO_NUM		72
	#define	RALINK_BTN_RST_GPIO_NUM		2
	#define	RALINK_BTN_WPS_GPIO_NUM		2
	#define	RALINK_MODE_SWITCH_GPIO1	40
	#define	RALINK_MODE_SWITCH_GPIO2	42
#else
#endif		/* CONFIG_RALINK_GPIO_WA855RE */


/*
#3	PRIVATE FUNTIONS
*/
/* === NONE === */


/*
#4	EXTERNALIZED FUNCTIONS
*/
/*
**	set some pins to gpio mode.
*/
void ra_gpio_product_gpiomode_init(u32 *mode)
{
	#if defined (CONFIG_RALINK_GPIO_WA855RE)
		(*mode) |= RALINK_GPIOMODE_WLED;		/* GPO72 <---> bit13 */
		(*mode) |= RALINK_GPIOMODE_EPHY;		/* GPIO40-44 <---> bit15 */

	#elif defined (CONFIG_RALINK_GPIO_WR810N)
		(*mode) |= RALINK_GPIOMODE_WLED;		/* GPO72 <---> bit13 */
		(*mode) |= RALINK_GPIOMODE_I2C;		/* GPIO2 <---> bit0 */
		(*mode) |= RALINK_GPIOMODE_EPHY;		/* GPIO40-44 <---> bit15 */
	#elif defined (CONFIG_RALINK_GPIO_E2_EU)
		/*
		**	Do it later
		*/
	#elif defined (CONFIG_RALINK_GPIO_C20I)
		/*
		**	Do it later
		*/
	#else
		#error not_defined
	#endif
}


/*
**	For Led initialization.
*/
#if defined (CONFIG_RALINK_GPIO_WA855RE)
/*
**	For wa855, the red led should quickly binking when system up.
@@	By TengFei, 13/04/26
*/
void ra_gpio_product_led_init(ralink_gpio_led_info* pLed)
{
	static int is_inited = false;
	ralink_gpio_led_info* thisLed = pLed + RALINK_LED_RED_GPIO_NUM;
	unsigned long tmp;

	if (is_inited)
	{
		return;
	}

	thisLed->gpio = RALINK_LED_RED_GPIO_NUM;
	thisLed->on = 1;	/* 1 tick */
	thisLed->off = 1;	/* 1 tick */
	thisLed->blinks = RALINK_GPIO_LED_INFINITY;	/* always blinking */
	thisLed->rests = 1;
	thisLed->times = RALINK_GPIO_LED_INFINITY;

	/*
	**	Attention! RALINK_LED_RED_GPIO_NUM is between 40 and 71
	*/
	tmp = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIO7140DIR));
	tmp |= RALINK_GPIO((RALINK_LED_RED_GPIO_NUM -40));
	*(volatile u32 *)(RALINK_REG_PIO7140DIR) = tmp;

	//printk("OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO\n");

	is_inited = true;
}
#elif defined (CONFIG_RALINK_GPIO_WR810N)
void ra_gpio_product_led_init(ralink_gpio_led_info* pLed)
{
	static int is_inited = false;
	ralink_gpio_led_info* thisLed = pLed + RALINK_LED_SYS_GPIO_NUM;
	unsigned long tmp;

	if (is_inited)
	{
		return;
	}

	thisLed->gpio = RALINK_LED_SYS_GPIO_NUM;
	thisLed->on = RALINK_GPIO_LED_INFINITY;	/* always on */
	thisLed->off = 0;
	thisLed->blinks = 0;
	thisLed->rests = 1;
	thisLed->times = RALINK_GPIO_LED_INFINITY;

	/*
	**	Attention! RALINK_LED_RED_GPIO_NUM is between 40 and 71
	*/
	tmp = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIO72DIR));
	tmp |= RALINK_GPIO((RALINK_LED_SYS_GPIO_NUM -72));
	*(volatile u32 *)(RALINK_REG_PIO72DIR) = tmp;

	is_inited = true;
}
#elif defined (CONFIG_RALINK_GPIO_E2_EU)
void ra_gpio_product_led_init(ralink_gpio_led_info* pLed)
{
		/*
		**	Do it later
		*/
}
#elif defined (CONFIG_RALINK_GPIO_C20I)
void ra_gpio_product_led_init(ralink_gpio_led_info* pLed)
{
		/*
		**	Do it later
		*/
}
#else
	#error not present
#endif

/*
**	set some gpio dir.
*/
void ra_gpio_product_gpiodir_init(void)
{
#if defined (CONFIG_RALINK_GPIO_WR810N)
	u32 gpiodir;

	gpiodir = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIO7140DIR));
	gpiodir &= ~ RALINK_GPIO((RALINK_MODE_SWITCH_GPIO1 - 40));
	//gpiodir &= (~ RALINK_GPIO(RALINK_MODE_SWITCH_GPIO1 - 40 + 1));
	gpiodir &= ~ RALINK_GPIO((RALINK_MODE_SWITCH_GPIO2 - 40));
	//gpiodir &= (~ RALINK_GPIO(RALINK_MODE_SWITCH_GPIO2 - 40 + 1));

	*(volatile u32 *)(RALINK_REG_PIO7140DIR) = gpiodir;
#endif
}

int ra_gpio_product_enirq(void)
{
	u32 i, gpioirq_r, gpioirq_f, gpioirq;
	
	/* For GPIO 0 - 23 */
	/* raise edge interrupt */
	gpioirq_r = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIORENA));
	/* falling edge interrupt */
	gpioirq_f = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIOFENA));
	for (i = 0; i <= 23; i++)
	{
		if (i == RALINK_BTN_RST_GPIO_NUM ||
			i == RALINK_BTN_WPS_GPIO_NUM)
		{
			gpioirq_r |= (1 << i);
			gpioirq_f |= (1 << i);
		}
	}
	*(volatile u32 *)(RALINK_REG_PIORENA) = cpu_to_le32(gpioirq_r);
	*(volatile u32 *)(RALINK_REG_PIOFENA) = cpu_to_le32(gpioirq_f);
	/* End for GPIO 0 - 23 */

	/* For GPIO 40 - 71 */
	/* raise edge interrupt */
	gpioirq_r = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIO7140RENA));
	/* falling edge interrupt */
	gpioirq_f = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIO7140FENA));
	for (i = 40; i <= 71; i++)
	{
		if (i == RALINK_BTN_RST_GPIO_NUM ||
			i == RALINK_BTN_WPS_GPIO_NUM)
		{
			gpioirq_r |= (1 << (i - 40));
			gpioirq_f |= (1 << (i - 40));
		}
	}
	*(volatile u32 *)(RALINK_REG_PIO7140RENA) = cpu_to_le32(gpioirq_r);
	*(volatile u32 *)(RALINK_REG_PIO7140RENA) = cpu_to_le32(gpioirq_f);
	/* For GPIO 40 - 71 */

#ifdef	RALINK_MODE_SWITCH_FUNC
	/* configure sysmode switch */
	/* raise edge interrupt */
	gpioirq_r = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIO7140RENA));
	gpioirq_r |= (1 << (RALINK_MODE_SWITCH_GPIO1 - 40));
	gpioirq_r |= (1 << (RALINK_MODE_SWITCH_GPIO2 -40));
	*(volatile u32 *)(RALINK_REG_PIO7140RENA) = cpu_to_le32(gpioirq_r);
	/* falling edge interrupt */
	gpioirq_f = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIO7140FENA));
	gpioirq_f |= (1 << (RALINK_MODE_SWITCH_GPIO1 - 40));
	gpioirq_f |= (1 << (RALINK_MODE_SWITCH_GPIO2 - 40));
	*(volatile u32 *)(RALINK_REG_PIO7140FENA) = cpu_to_le32(gpioirq_f);
#endif	/* RALINK_MODE_SWITCH_FUNC */
	
	/* enable global gpio interrupt */
	gpioirq = le32_to_cpu(*(volatile u32 *)(RALINK_REG_INTENA));
	gpioirq |= cpu_to_le32(RALINK_INTCTL_PIO);
	*(volatile u32 *)(RALINK_REG_INTENA) = gpioirq;
	return 0;
}


#ifdef	RALINK_GPIO_PROC_FS
#include <linux/proc_fs.h>		/* procfs */

static struct proc_dir_entry *simple_cfg_entry = NULL;

#ifdef	RALINK_MODE_SWITCH_FUNC
static struct proc_dir_entry *sysmode_entry = NULL;

/*
**	return sysmode which read gpio value from RALINK_REG_PIO7140DATA
*/
static unsigned ra_gpio_sysmode_read(void)
{
	unsigned data = 0x0;
	unsigned sysmod = SYSTEM_MODE_INV;

	data = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIO7140DATA));
	data &= ((1 << (RALINK_MODE_SWITCH_GPIO1 - 40)) | (1 << (RALINK_MODE_SWITCH_GPIO2 - 40)));
	
	switch (data)
	{
	case 0x1:
		sysmod = SYSTEM_MODE_NORMAL_ROUTER; 
		break;
	case 0x4:
		sysmod = SYSTEM_MODE_REPEATER;
		break;
	case 0x5:
		sysmod = SYSTEM_MODE_CLIENT; 
		break;

	default:
		printk("Error: No such sysmode(data = %d), set it as default router mode\n", data);
		sysmod = SYSTEM_MODE_NORMAL_ROUTER; 
	}

	return sysmod;
}

unsigned ra_gpio_sysmode_gset(int set, unsigned mode)
{
	static unsigned product_sysmode = SYSTEM_MODE_INV;

	if (set)
	{
		product_sysmode = mode;
	}
	return product_sysmode;
}

int ra_gpio_sysmode_chged(void)
{
	unsigned old_mod, curr_mod;

	old_mod = ra_gpio_sysmode_gset(0, 0);
	curr_mod = ra_gpio_sysmode_read();

	if (old_mod == curr_mod)
		return 0;
	
#ifdef	CONFIG_RALINK_GPIO_WR810N_EU
	/* For wr810n_eu, router mode contains AP mode */
	if (curr_mod == SYSTEM_MODE_NORMAL_ROUTER && old_mod == SYSTEM_MODE_AP)
		return 0;
#endif	/* CONFIG_RALINK_GPIO_WR810N_EU  */

	ra_gpio_sysmode_gset(1, curr_mod);
	return 1;
}


static int ralink_gpio_sysmode_read(char *page, char **start, off_t off,
               int count, int *eof, void *data)
{
	int system_mode = ra_gpio_sysmode_gset(0, SYSTEM_MODE_INV);
	
	return sprintf(page, "%d\n", system_mode);
}

static int ralink_gpio_sysmode_write(struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	u_int32_t val = 0;

	if (sscanf(buf, "%d", &val) != 1)
	{
		return -EINVAL;
	}
	if (val <= SYSTEM_MODE_INV || val > SYSTEM_MODE_MAX)
	{
		return -EINVAL;
	}

	ra_gpio_sysmode_gset(1, val);        /* set system mode from web server */
	return count;
}

#endif	/* RALINK_MODE_SWITCH_FUNC */

/*
**	create proc files for sysmode
*/
int ralink_gpio_proc_init(void)
{
	if (simple_cfg_entry != NULL)
	{
		printk ("Already have a proc entry for /proc/simple_config!\n");
		return -ENOENT;
	}

	simple_cfg_entry = proc_mkdir("simple_config", NULL);
	if (!simple_cfg_entry)
	{
		return -ENOENT;
	}

#ifdef	RALINK_MODE_SWITCH_FUNC
	sysmode_entry = create_proc_entry("sysmode", 0666,
								simple_cfg_entry);
	if (!sysmode_entry)
	{
		return -ENOENT;
	}
	sysmode_entry->read_proc = ralink_gpio_sysmode_read;
	sysmode_entry->write_proc = ralink_gpio_sysmode_write; 

	ra_gpio_sysmode_gset(1, ra_gpio_sysmode_read());
#endif	/* RALINK_MODE_SWITCH_FUNC */
	
	return 0;
}

#if	0/*SYSMODE_HW_CONTROL*/

#include <linux/timer.h>
static struct timer_list gpio_sysmode_timer;


static int gpio_detect_sysmode(u32 unused)
{
	int sysmode;
	u32 gpio_data, tmp;

	ra_gpio_product_gpiodir_init();
	gpio_data = le32_to_cpu(*(volatile u32 *)(RALINK_REG_PIODATA));
	
	tmp = gpio_data & ((1 << ( RALINK_MODE_SWITCH_GPIO1 - 40)) | (1 << (RALINK_MODE_SWITCH_GPIO2 - 40)));
	switch (tmp)
	{
	case 0x1:
		sysmode = SYSTEM_MODE_NORMAL_ROUTER;
		break;

	case 0x4:
		sysmode = SYSTEM_MODE_REPEATER;
		break;

	case 0x5:
		sysmode = SYSTEM_MODE_CLIENT;
		break;

	default:
		/* nothing to do */
		;
	}

	if (product_sysmode != SYSTEM_MODE_INV && product_sysmode != sysmode)
	{
		printk("(newmode, oldmode) <=> (%d, %d)\n", sysmode, product_sysmode);
		product_sysmode = sysmode;
		return 0;
	}

	mod_timer(&gpio_sysmode_timer, jiffies + 1 * HZ);
}

int ralink_gpio_sysmode_probe(void)
{
	ra_gpio_product_gpiodir_init();
	//init_timer(&gpio_sysmode_timer);
	//gpio_sysmode_timer.function = gpio_detect_sysmode;
	//mod_timer(&gpio_sysmode_timer, jiffies + 3 * HZ);

	return 0;
}
#endif	/* SYSMODE_HW_CONTROL */
#endif	/* RALINK_GPIO_PROC_FS */

#endif	/* PRODUCT_GPIO_FEATURE */
 
