/******************************************************************************
 *
 * Copyright (c) 2013 TP-LINK Technologies CO.,LTD.
 * All rights reserved.
 *
 * FILE NAME  :   tp_gpio_event.c
 * VERSION    :   1.0
 * DESCRIPTION:   Source file to implement GPIO event detection.
 *
 * AUTHOR     :   Teng Fei <tengfei@tp-link.net>
 * CREATE DATE:   02/10/2014
 *
 * HISTORY    :
 * 01   06/19/2013  Teng Fei     Create.
 *
******************************************************************************/

#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/completion.h>
#include <linux/sched.h>
#include <linux/list.h>
#include <linux/slab.h>
#include <linux/ioctl.h>


#include <linux/kthread.h>
#include <linux/mutex.h>
#include <linux/sched.h>	/* task_struct */
#include <linux/reboot.h>	/* kernel_restart */

#ifdef	CONFIG_MTD_SPI_RALINK
#include <linux/mtd/partitions.h>
#include <linux/mtd/mtd.h>
#include <linux/spi/spi.h>
#include <linux/spi/flash.h>
#include <linux/spinlock.h>
#include "../mtd/maps/ralink-flash.h"		/* MTD_BOOTLDR_PART_SIZE etc. */
#endif	/* CONFIG_MTD_SPI_RALINK */

#include "tp_gpio_event.h"
#include "ralink_gpio.h"

static struct task_struct *kgpiod_task;
static struct task_struct *kgpiod_task_system_led;
u32 system_led_blink = 1;
/* Wakes up kgpiod */
static DECLARE_WAIT_QUEUE_HEAD(kgpiod_wait);

#ifdef	CONFIG_MTD_SPI_RALINK
#include <linux/semaphore.h>
struct flash_info {
    struct semaphore    lock;
    struct mtd_info     mtd;
    struct chip_info    *chip;
    u8     command[5];
};
#endif	/* CONFIG_MTD_SPI_RALINK */

static void gpio_factory_restore(void)
{
#ifdef	CONFIG_MTD_SPI_RALINK
	extern struct flash_info *flash;
	extern int ramtd_erase(struct mtd_info *mtd, struct erase_info *instr);

	struct flash_info *ralink_flash = flash;
	struct mtd_info *mtd = &ralink_flash->mtd;
	struct erase_info *erase  = NULL;

	spinlock_t gpio_flash_spinlock = SPIN_LOCK_UNLOCKED;
	spin_lock_init(&gpio_flash_spinlock);

	 /* disable interrupt */
	//spin_lock_irqsave(&gpio_flash_spinlock, 0);

	erase = (struct erase_info *)kmalloc(sizeof(struct erase_info), GFP_KERNEL);
	if (!erase)
	{
		printk("Erase failed, Alloc erase-buffer(%p) failed!\n", erase);
		goto end;
	}

	erase->addr = MTD_BOOTLDR_PART_SIZE + MTD_KERNEL_PART_SIZE + MTD_ROOT_FS_PART_SIZE;
	erase->len = MTD_USRCFG_PART_SIZE;
	erase->mtd = mtd;
	erase->callback = NULL;
	erase->priv = 0;
	if (ramtd_erase(mtd, erase)) 
	{
		printk("Erase failed, call mtd->erase failed, Ignore it!\n");
	}

end:
	//spin_unlock_irqrestore(&gpio_flash_spinlock, 0);
	printk("exit!\n");
#endif	/* CONFIG_MTD_SPI_RALINK */
}


#if defined(CONFIG_MT7610_AP) || defined(CONFIG_MT7610_AP_APCLI)
#include <linux/netdevice.h>	/* dev_get_by_name */
extern void rtnl_lock(void);
extern void rtnl_unlock(void);
static void gpio_close_netdev(void)
{
	struct net_device* netDev = NULL;
	const char* ifname[] = {"rai0", "apclii0", "ra0", "apcli0", ""};
	char** curr_ifname = ifname;

	while (curr_ifname && *curr_ifname && **curr_ifname)
	{
		printk("GPIO_%s: ifname_%s\n", __func__, *curr_ifname);
		netDev = dev_get_by_name(&init_net, *curr_ifname);
		if (netDev)
		{
			rtnl_lock();
			if (!dev_close(netDev))
				printk("GPIO_%s: close ifname_%s successful!\n", __func__, *curr_ifname);
			rtnl_unlock();
			dev_put(netDev);
		}
		curr_ifname++;
	}
}
#endif	/* CONFIG_MT7610_AP || CONFIG_MT7610_AP_APCLI */

static void gpio_events_handle(void)
{
	unsigned events = gpio_event_gset(TP_GPIO_GET_EVT, 0, 0);
	
	if (events == TP_GPIO_EVT_NULL)
	{
		printk("NO GPIO Event!\n");
		return;
	}

	printk("GPIO Event(0x%x) Happened!\n", events);
	if (events & TP_GPIO_EVT_RST)
	{
	#if defined(CONFIG_MT7610_AP) || defined(CONFIG_MT7610_AP_APCLI)
		gpio_close_netdev();
		schedule_timeout_interruptible(1*HZ);
	#endif	/* CONFIG_MT7610_AP || CONFIG_MT7610_AP_APCLI */

		printk("Need erase uc block!\n");
		gpio_factory_restore();
		kernel_restart(NULL);
	}

	if (events & TP_GPIO_EVT_SYSMOD)
	{
	#ifdef	RALINK_MODE_SWITCH_FUNC
		/*  First check IF system mode is really changed. */	
		if (!ra_gpio_sysmode_chged())
		{
			goto end;
		}
	#endif
		kernel_restart(NULL);
	}

end:
	/* reset gpio_event */
	gpio_event_gset(TP_GPIO_SET_EVT, TP_GPIO_EVT_NULL, 0);
}

static int gpio_thread(void *__unused)
{
	do {
		gpio_events_handle();
		wait_event_interruptible(kgpiod_wait,
				gpio_event_gset(TP_GPIO_GET_EVT, 0, 0) || kthread_should_stop());
	} while (!kthread_should_stop());

	pr_debug("GPIO: kgpiod exiting\n");
	return 0;
}


unsigned gpio_event_gset(const int set, const unsigned event, const int do_wake)
{
	static unsigned gpio_events = TP_GPIO_EVT_NULL;

	if (set)
	{
		if (event == TP_GPIO_EVT_NULL)
			gpio_events = event;
		else
			gpio_events |= event;

		if (do_wake)
			wake_up(&kgpiod_wait);
	}
	return gpio_events;
}

int gpio_thread_init(void)
{
	kgpiod_task = kthread_run(gpio_thread, NULL, "kgpiod");
	if (!IS_ERR(kgpiod_task))
		return 0;

	printk(KERN_ERR "GPIO: can't start kgpiod\n");
	return -1;
}

void gpio_thread_exit(void)
{
	if (!IS_ERR(kgpiod_task))
	{
		kthread_stop(kgpiod_task);
	}
}


extern int gpio_power_system_led_thread(void *__unused);
int gpio_thread_led_init(void)
{
	kgpiod_task_system_led = kthread_run(gpio_power_system_led_thread, NULL, "kledd");
	if (!IS_ERR(kgpiod_task_system_led))
		return 0;

	printk(KERN_ERR "GPIO: can't start kgpiod\n");
	return -1;    
}

void gpio_thread_led_exit(void)
{
	if (!IS_ERR(kgpiod_task_system_led))
	{
		kthread_stop(kgpiod_task_system_led);
	}       
}

