/***************************************************************
Copyright (c), 2013, TP-LINK Technologies CO.,LTD.

File name:	tp_mt7620_gpio.c
Version:		1.0
Description:	Ralink MT7620 gpio driver.
	
Author:		Xiongliuzhong
Create Date:	2013-3-21

History:		01, 21Mar2013, xiongliuzhong     Create.
			02, 06Nov2013, tengfei		rename source/header file and 
									provide kernel config item support to
									adapt to requirements for different 
									products.
			03, 07Nov2013, tengfei		provide gpio control for 5G wireless led.
***************************************************************/
#include <linux/init.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/interrupt.h>
#include <linux/fs.h>
#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/delay.h>
#include <linux/proc_fs.h>
#include <asm/uaccess.h>
#include "tp_mt7620_gpio.h"

#include <linux/kthread.h>

#include <asm/rt2880/surfboardint.h>

#ifdef	CONFIG_TP_GPIO_EVENT
#include "tp_gpio_event.h"
#endif	/* CONFIG_TP_GPIO_EVENT */

#define NAME						"ralink_gpio"
#define RALINK_GPIO_DEVNAME		"gpio"
static u32 mt7620_gpio_intp = 0;
static u32 mt7620_gpio_edge = 0;

static int mt7620_gpio_factory_reset_inited = 0;
static unsigned int mt7620_gpio_rst_time = 0;
static struct timer_list mt7620_gpio_rst_timer;
struct timer_list mt7620_gpio_sysMode_timer;

#ifdef GPIO_WPS_BIT_CONTROL
static u32 mt7620_gpio_wps_intp = 0;
static u32 mt7620_gpio_wps_edge = 0;
static struct timer_list mt7620_gpio_wps_timer;
#endif

#ifdef GPIO_POWER_ONOFF_CONTROL
static u32 mt7620_gpio_power_onoff_intp = 0;
static u32 mt7620_gpio_power_onoff_edge = 0;
static struct timer_list mt7620_gpio_power_onoff_timer;
static struct timer_list mt7620_gpio_power_blink_timer;
int mt7620_gpio_power_onoff_status = 1;  /* 1:power on, 0:power off */
static struct proc_dir_entry *power_onoff_entry = NULL;
#endif

#ifdef LED_POWER_SWITCH_ON_OFF
static u32 mt7620_gpio_led_onoff_intp = 0;
static u32 mt7620_gpio_led_onoff_edge = 0;
static struct timer_list mt7620_gpio_led_onoff_timer;
/* g_all_led_power_on_off for current status */
#endif

int mt7620_gpio_reset_enable = 0;			/* 1 is reset_enable, 0 is reset not working */
int mt7620_gpio_mode_switch_enable = 0;	/* 1 is mode switch enable, 0 is switch mode */
int mt7620_gpio_block_wps = 1;			/* 1 is block_wps, 0 is wps processing */
int g_sysMode = 0;
int g_isSoftSysModeEnable = 0;
int g_wpsProcessPid = -1;
int g_internetLedStatus = 0;	/* 0: internet led off; 1: on for 3G; 2: on for WISP; 3: on for wired connection */

struct proc_dir_entry *simple_config_entry = NULL;

static struct proc_dir_entry *system_mode_entry = NULL;

static struct proc_dir_entry *wps_process_pid_entry = NULL;

#ifdef GPIO_ETHERNET_LED
static int g_ethernet_led = 0;
static struct proc_dir_entry *ethernet_led_entry = NULL;
#endif

#ifdef GPIO_RANGE_EXTENDER_LED
static int g_range_extender_led = 0;
static struct proc_dir_entry *range_extender_led_entry = NULL;
#endif
#ifdef GPIO_RANGE_EXTENDER_TRICOLOR_LED
static int g_range_extender_tricolor_led = 10;
static struct proc_dir_entry *range_extender_tricolor_led_entry = NULL;
#endif



#ifdef GPIO_2G_LED
static int g_wlan_2g_led = 0;
static struct proc_dir_entry *wlan_2g_led_entry = NULL;
#endif
#ifdef WLAN_2G_GPIO_TRICOLOR_LED
static int g_wlan_2g_tricolor_led = 0;
static struct proc_dir_entry *wlan_2g_tricolor_led_entry = NULL;
#endif


#ifdef GPIO_5G_LED
static int g_wlan_5g_led = 0;
static struct proc_dir_entry *wlan_5g_led_entry = NULL;
#endif
#ifdef WLAN_5G_GPIO_TRICOLOR_LED
static int g_wlan_5g_tricolor_led = 0;
static struct proc_dir_entry *wlan_5g_tricolor_led_entry = NULL;
#endif	/* WLAN_5G_GPIO_TRICOLOR_LED */

#ifdef GPIO_WLAN_BACK_END_POST_LED
int g_wlan_back_end_led = 0;
static struct proc_dir_entry *wlan_back_end_led_entry = NULL;
#endif

#ifdef LED_POWER_SWITCH_ON_OFF
int g_all_led_power_on_off = 1;
static struct proc_dir_entry *all_led_power_on_off_entry = NULL;
#endif

int g_power_system_led = 0;
extern u32 system_led_blink;
#ifdef GPIO_POWER_SYSTEM_LED
static struct proc_dir_entry *power_system_led_entry = NULL;
static struct proc_dir_entry *power_system_led_blink_entry = NULL;
#endif


EXPORT_SYMBOL(simple_config_entry);

#ifdef LED_POWER_SWITCH_ON_OFF
#if defined(CONFIG_RALINK_GPIO_RE210)
static void mt7620_gpio_set_all_led(int onoff);
static void mt7620_gpio_check_power_blink(unsigned long nothing);
#endif
#endif

/**************************************************************
Function:		static u32 mt7620_gpio_set_bit(u32 pos, u32 bit, u32 in_value)
Description:		set the in_value's bit of pos as "u32 bit"
Input:			pos, in_value, bit
Output:			NA
return:			out_value
***************************************************************/
static u32 mt7620_gpio_set_bit(u32 pos, u32 bit, u32 in_value)
{
	u32 mask = 1 << pos;
	u32 out_value = in_value;
	if (bit)
		out_value |= mask;
	else
		out_value &= ~mask;

	return out_value;
}

/**************************************************************
Function:		static u32 mt7620_gpio_is_set(u32 pos, u32 in_value)
Description:		test the in_value's bit of pos  
Input:			pos, in_value
Output:			NA
return:			if the pos bit is 1, return 1(TRUE), else return 0
***************************************************************/
static u32 mt7620_gpio_is_set(u32 pos, u32 in_value)
{
	u32 mask = 1 << pos;
    
    return in_value &= mask;
}

static u32 mt7620_gpio_read(u32 addr)
{
	u32 value;
	
	value = le32_to_cpu(*(volatile u32 *)(addr));
	
	return value;
}

static int mt7620_gpio_write(volatile u32 addr, u32 value)
{
	*(volatile u32 *)(addr) = cpu_to_le32(value);

	return 0;
}

static int mt7620_gpio_reg_offset(u32 value)
{	 
	u32 ret = 0;
	
	if (value<= 23)
	{
		ret = value;
	}
	else if (value >=24 && value<= 39)
	{
		ret = value - 24;
	}
	else if (value >=40 && value<= 71)
	{
		ret = value - 40;
	}
	else if(value >= 72)
	{
		ret = value - 72;
	}
	
	return ret;
}

static u32 mt7620_gpio_setdir_api(u32 gpionum, u32 dirValue)
{
    u32 gpiodir, diraddr;

    gpiodir = 0;

    if (gpionum >=0 && gpionum <=23)
    {
        diraddr = TP_MT7620_REG_PIODIR;
    }
    else if (gpionum >=24 && gpionum <= 39)
    {
        diraddr = TP_MT7620_REG_PIO3924DIR;
    }
    else if (gpionum >=40 && gpionum <= 71)
    {
        diraddr = TP_MT7620_REG_PIO7140DIR;
    }
    else if (gpionum == 72)
    {
        diraddr = TP_MT7620_REG_PIO72DIR;
    }
    else
    {
        return gpiodir;
    }

    gpiodir = mt7620_gpio_read(diraddr);
	gpiodir = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(gpionum),
					   dirValue,
					   gpiodir);    
    mt7620_gpio_write(diraddr, gpiodir);
        
    return gpiodir;
}


static int mt7620_gpio_setmode(void)
{
	u32 gpiomode = 0;
	
	gpiomode = mt7620_gpio_read(TP_MT7620_REG_GPIOMODE);

#ifdef GPIO_WLAN_LED
	/* set bit[13] WLAN_N, as gpio mode */
	gpiomode = mt7620_gpio_set_bit(TP_MT7620_GPIOMODE_WLED_BIT_13,
								 TP_MT7620_GPIO_VALUE_SET,
								 gpiomode);
#endif

	/* Note here by xiongliuzhong, 28Mar13, i2c mode is bit0 in datasheet, gpio mode */
	gpiomode = mt7620_gpio_set_bit(TP_MT7620_GPIOMODE_I2C_BIT_0,
					   TP_MT7620_GPIO_VALUE_SET,
					   gpiomode);

	/* set bit[2-4] UARTF, GPIO7-14 as gpio */
	gpiomode = mt7620_gpio_set_bit(TP_MT7620_GPIOMODE_UARTF_BIT_2, 
					   TP_MT7620_GPIO_VALUE_SET,
					   gpiomode);
	gpiomode = mt7620_gpio_set_bit(TP_MT7620_GPIOMODE_UARTF_BIT_3,
					   TP_MT7620_GPIO_VALUE_SET,
					   gpiomode);
	gpiomode = mt7620_gpio_set_bit(TP_MT7620_GPIOMODE_UARTF_BIT_4,
					   TP_MT7620_GPIO_VALUE_SET,
					   gpiomode);

	gpiomode = mt7620_gpio_set_bit(TP_MT7620_GPIOMODE_RGMII1_BIT_9,
					   TP_MT7620_GPIO_VALUE_SET,
					   gpiomode);

#ifdef GPIO_ETHERNET_LED
	gpiomode = mt7620_gpio_set_bit(TP_MT7620_GPIOMODE_EPHY_LED_BIT_15,
					   TP_MT7620_GPIO_VALUE_SET,
					   gpiomode);
#endif

#ifdef WLAN_2G_GPIO_TRICOLOR_LED
	gpiomode = mt7620_gpio_set_bit(TP_MT7620_GPIOMODE_WLED_BIT_13,
								 TP_MT7620_GPIO_VALUE_SET,
								 gpiomode);
#endif
#ifdef GPIO_2G_LED
	gpiomode = mt7620_gpio_set_bit(GPIO_2G_LED_GPIOMODE_BIT,
					   GPIO_2G_LED_GPIOMODE_BIT_VALUE,
					   gpiomode);
#endif

#ifdef GPIO_5G_LED
	gpiomode = mt7620_gpio_set_bit(GPIO_5G_LED_GPIOMODE_BIT,
					   GPIO_5G_LED_GPIOMODE_BIT_VALUE,
					   gpiomode);
#endif

#ifdef GPIO_RANGE_EXTENDER_LED_BIT_CONTROL
	if (GPIO_RANGE_EXTENDER_LED_BIT_CONTROL <= 39 &&
		GPIO_RANGE_EXTENDER_LED_BIT_CONTROL >= 37)
	{
		gpiomode = mt7620_gpio_set_bit(TP_MT7620_GPIOMODE_WLED_BIT_12,
								 TP_MT7620_GPIO_VALUE_SET,
								 gpiomode);
	}
#endif

#ifdef GPIO_RANGE_EXTENDER_TRICOLOR_LED
	gpiomode = mt7620_gpio_set_bit(GPIO_RANGE_EXTENDER_TRICOLOR_LED_GPIOMODE_BIT,
					   GPIO_RANGE_EXTENDER_TRICOLOR_LED_GPIOMODE_BIT_VALUE,
					   gpiomode);
#endif

#ifdef GPIO_POWER_SYSTEM_LED
	gpiomode = mt7620_gpio_set_bit(GPIO_POWER_SYSTEM_LED_GPIOMODE_BIT,
					   GPIO_POWER_SYSTEM_LED_GPIOMODE_BIT_VALUE,
					   gpiomode);
#endif

	mt7620_gpio_write(TP_MT7620_REG_GPIOMODE, gpiomode);

	return 0;
}



static int mt7620_gpio_setdir(void)
{
#ifdef GPIO_RESET_FAC_CONTROL
    mt7620_gpio_setdir_api(GPIO_RESET_FAC_CONTROL, GPIO_RESET_FAC_CONTROL_DIRVALUE);
#endif

#ifdef WLAN_2G_GPIO_TRICOLOR_LED
    mt7620_gpio_setdir_api(WLAN_2G_GPIO_TRICOLOR_LED_BIT_CONTROL0, 
        WLAN_2G_GPIO_TRICOLOR_LED_BIT_CONTROL0_DIRVALUE);
    mt7620_gpio_setdir_api(WLAN_2G_GPIO_TRICOLOR_LED_BIT_CONTROL1, 
        WLAN_2G_GPIO_TRICOLOR_LED_BIT_CONTROL1_DIRVALUE);
#endif	

#ifdef GPIO_2G_LED
    mt7620_gpio_setdir_api(GPIO_2G_LED_BIT_CONTROL, 
        GPIO_2G_LED_BIT_CONTROL_DIRVALUE);
#endif

#ifdef GPIO_5G_LED
    mt7620_gpio_setdir_api(GPIO_5G_LED_BIT_CONTROL, 
        GPIO_5G_LED_BIT_CONTROL_DIRVALUE);
#endif

#ifdef GPIO_WPS_BIT_CONTROL
    mt7620_gpio_setdir_api(GPIO_WPS_BIT_CONTROL, 
        GPIO_WPS_BIT_CONTROL_DIRVALUE);
#endif

#ifdef GPIO_RANGE_EXTENDER_LED
    mt7620_gpio_setdir_api(GPIO_RANGE_EXTENDER_LED_BIT_CONTROL, 
        GPIO_RANGE_EXTENDER_LED_BIT_CONTROL_DIRVALUE);
#endif

#ifdef GPIO_RANGE_EXTENDER_TRICOLOR_LED
    mt7620_gpio_setdir_api(GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL0, 
        GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL0_DIRVALUE);
    mt7620_gpio_setdir_api(GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL1, 
        GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL1_DIRVALUE);
#endif

#ifdef GPIO_WLAN_BACK_END_POST_LED
    mt7620_gpio_setdir_api(GPIO_WLAN_BACK_END_LED_BIT_CONTROL, 
        GPIO_WLAN_BACK_END_LED_BIT_CONTROL_DIRVALUE);
#endif 

#ifdef LED_POWER_SWITCH_ON_OFF
    mt7620_gpio_setdir_api(GPIO_LED_POWER_SWITCH_BIT_CONTROL, 
        GPIO_LED_POWER_SWITCH_BIT_CONTROL_DIRVALUE);
#endif 

#ifdef GPIO_ETHERNET_LED
    mt7620_gpio_setdir_api(GPIO_ETHERNET_LED_BIT_CONTROL, 
        GPIO_ETHERNET_LED_BIT_CONTROL_DIRVALUE);
#endif

#ifdef GPIO_POWER_ONOFF_CONTROL
    mt7620_gpio_setdir_api(GPIO_POWER_ONOFF_CONTROL, 
        GPIO_POWER_ONOFF_CONTROL_DIRVALUE);
#endif

#ifdef GPIO_POWER_SYSTEM_LED
    mt7620_gpio_setdir_api(GPIO_POWER_SYSTEM_LED_BIT_CONTROL, 
        GPIO_POWER_SYSTEM_LED_BIT_CONTROL_DIRVALUE);
#endif

	return 0;
}

#if 		0
static int mt7620_gpio_setpol(void)
{
	u32 gpiopol;

	/* config gpio0-23 */
	gpiopol = mt7620_gpio_read(TP_MT7620_REG_PIOPOL);

	gpiopol = mt7620_gpio_set_bit(GPIO_RESET_FAC_CONTROL, 
					   TP_MT7620_GPIO_VALUE_CLR, 
					   gpiopol);

#ifdef SUPPORT_HARDWARE_MULTI_MODE
	gpiopol = mt7620_gpio_set_bit(GPIO_MODE_SWITCH_CONTROL0,
					   TP_MT7620_GPIO_VALUE_CLR, 
					   gpiopol);

	gpiopol = mt7620_gpio_set_bit(GPIO_MODE_SWITCH_CONTROL1,
					   TP_MT7620_GPIO_VALUE_CLR,
				 	   gpiopol);
#endif

#ifdef GPIO_TRICOLOR_LED
	gpiopol = mt7620_gpio_set_bit(GPIO_TRICOLOR_LED_CONTROL0,
					   TP_MT7620_GPIO_VALUE_CLR,
					   gpiopol);

	gpiopol = mt7620_gpio_set_bit(GPIO_TRICOLOR_LED_CONTROL1,
				        TP_MT7620_GPIO_VALUE_CLR, 
					    gpiopol);
#endif

	mt7620_gpio_write(TP_MT7620_REG_PIOPOL, gpiopol);

	/* config gpio72 */
#ifdef GPIO_WLAN_LED
	gpiopol = mt7620_gpio_read(TP_MT7620_REG_PIO72POL);
	mt7620_gpio_set_bit(GPIO_WLAN_LED_CONTROL, 
					   TP_MT7620_GPIO_VALUE_CLR, 
					   gpiopol);	// wlan led on
	mt7620_gpio_write(TP_MT7620_REG_PIO72POL, gpiopol);
#endif

	return 0;
}


static int mt7620_gpio_setdata(void)
{
	u32 gpiodata;

	/* config gpio0-23 */
	gpiodata = mt7620_gpio_read(TP_MT7620_REG_PIODATA);
#ifdef GPIO_RESET_FAC_CONTROL
	if (GPIO_RESET_FAC_CONTROL <= 23)
	{
		gpiodata = mt7620_gpio_set_bit(GPIO_RESET_FAC_CONTROL,
								 TP_MT7620_GPIO_VALUE_SET,
								 gpiodata);
	}
#endif
#if 0
	/* mode switch data init */
#ifdef SUPPORT_HARDWARE_MULTI_MODE
	gpiodata = mt7620_gpio_set_bit(GPIO_MODE_SWITCH_CONTROL0, 
								 TP_MT7620_GPIO_VALUE_CLR, 
								 gpiodata);
	gpiodata = mt7620_gpio_set_bit(GPIO_MODE_SWITCH_CONTROL1, 
								 TP_MT7620_GPIO_VALUE_CLR, 
								 gpiodata);
#endif
#endif

/*************
* 9   10
* ON, ON  red
* off,ON  green (PCB left on, right off)
* ON, OFF red   (PCB left off, right on)
* off,off off
*************/
#ifdef WLAN_2G_GPIO_TRICOLOR_LED
	gpiodata = mt7620_gpio_set_bit(WLAN_2G_GPIO_TRICOLOR_LED_BIT_CONTROL0,
					   TP_MT7620_GPIO_VALUE_SET,
					   gpiodata);
#endif

#ifdef GPIO_WPS_BIT_CONTROL
	gpiodata = mt7620_gpio_set_bit(GPIO_WPS_BIT_CONTROL,
					   WLAN_2G_GPIO_TRICOLOR_LED_BIT_CONTROL_ON,
					   gpiodata);
#endif

#ifdef GPIO_TRICOLOR_LED
	/* led status data init */
	gpiodata = mt7620_gpio_set_bit(GPIO_TRICOLOR_LED_CONTROL0, TRICOLOR_LED_OFF, gpiodata);
	gpiodata = mt7620_gpio_set_bit(GPIO_TRICOLOR_LED_CONTROL1, TRICOLOR_LED_OFF, gpiodata);
#endif

#ifdef GPIO_INTERNET_LED
	gpiodata = mt7620_gpio_set_bit(GPIO_INTERNET_LED_CONTROL0, INTERNET_LED_OFF, gpiodata);
#endif

#ifdef GPIO_SYS_LED
        gpiodata = mt7620_gpio_set_bit(GPIO_SYS_LED_BIT_CONTROL, GPIO_SYS_LED_BIT_CONTROL_ON, gpiodata);
#endif

#ifdef GPIO_USB_POWER_SUPPORT
    gpiodata = mt7620_gpio_set_bit(GPIO_USB_POWER_SUPPORT, GPIO_USB_POWER_ON, gpiodata);
#endif

	gpiodata = mt7620_gpio_write(TP_MT7620_REG_PIODATA, gpiodata);

	/* config gpio39-24 */
	gpiodata = mt7620_gpio_read(TP_MT7620_REG_PIO3924DATA);

#ifdef GPIO_RESET_FAC_CONTROL
	if (GPIO_RESET_FAC_CONTROL <= 39 && GPIO_RESET_FAC_CONTROL >= 24)
	{
		gpiodata = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_RESET_FAC_CONTROL),
				TP_MT7620_GPIO_VALUE_SET, 
				gpiodata);
	}
#endif

#ifdef GPIO_RANGE_EXTENDER_LED
	gpiodata = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_RANGE_EXTENDER_LED_BIT_CONTROL), 
		GPIO_RANGE_EXTENDER_LED_BIT_CONTROL_ON, 
		gpiodata);
#endif

#ifdef GPIO_WLAN_BACK_END_POST_LED
	gpiodata = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_WLAN_BACK_END_LED_BIT_CONTROL), 
		GPIO_WLAN_BACK_END_LED_BIT_CONTROL_ON, 
		gpiodata);
#endif 

#ifdef LED_POWER_SWITCH_ON_OFF
	gpiodata = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_LED_POWER_SWITCH_BIT_CONTROL), 
		GPIO_LED_POWER_SWITCH_BIT_CONTROL_ON, 
		gpiodata);
#endif 

	mt7620_gpio_write(TP_MT7620_REG_PIO3924DATA, gpiodata);

	/* config gpio71-40*/
	gpiodata = mt7620_gpio_read(TP_MT7620_REG_PIO7140DATA);

#ifdef GPIO_ETHERNET_LED
	gpiodata = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_ETHERNET_LED_BIT_CONTROL), 
		GPIO_ETHERNET_LED_BIT_CONTROL_ON, 
		gpiodata);
#endif

	mt7620_gpio_write(TP_MT7620_REG_PIO7140DATA, gpiodata);

	/* config gpio72 */
#ifdef GPIO_WLAN_LED
	gpiodata = mt7620_gpio_read(TP_MT7620_REG_PIO72DATA);
	gpiodata = mt7620_gpio_set_bit(GPIO_WLAN_LED_CONTROL, WLAN_LED_ON, gpiodata);
	mt7620_gpio_write(TP_MT7620_REG_PIO72DATA, gpiodata);
#endif

#ifdef WLAN_2G_GPIO_TRICOLOR_LED
	gpiodata = mt7620_gpio_read(TP_MT7620_REG_PIO72DATA);
	gpiodata = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(WLAN_2G_GPIO_TRICOLOR_LED_BIT_CONTROL1), 
		WLAN_2G_GPIO_TRICOLOR_LED_BIT_CONTROL_ON, 
		gpiodata);
	mt7620_gpio_write(TP_MT7620_REG_PIO72DATA, gpiodata);
#endif	
	return 0;
}
#endif	/* #if 0 */

/*
 * 1. save the PIOINT and PIOEDGE value
 * 2. clear PIOINT by writing 1
 * (called by interrupt handler)
 */
static int mt7620_gpio_save_clear_intp(void)
{
#ifdef GPIO_RESET_FAC_CONTROL
	if (GPIO_RESET_FAC_CONTROL <= 23)
	{
		mt7620_gpio_intp = mt7620_gpio_read(TP_MT7620_REG_PIOINT);	//read, whether the chanaged of interrupt
		mt7620_gpio_edge = mt7620_gpio_read(TP_MT7620_REG_PIOEDGE);	//read, raise0 is falling
		mt7620_gpio_write(TP_MT7620_REG_PIOINT, 
						 (mt7620_gpio_intp | TP_MT7620_GPIO(GPIO_RESET_FAC_CONTROL)));	//write 1 clear intp
		mt7620_gpio_write(TP_MT7620_REG_PIOEDGE, 
						 (mt7620_gpio_edge | TP_MT7620_GPIO(GPIO_RESET_FAC_CONTROL)));	//write 1 clear 
	}
	else if (GPIO_RESET_FAC_CONTROL <= 39 && GPIO_RESET_FAC_CONTROL >= 24)
	{
		mt7620_gpio_intp = mt7620_gpio_read(TP_MT7620_REG_PIO3924INT);	//read, whether the chanaged of interrupt
		mt7620_gpio_edge = mt7620_gpio_read(TP_MT7620_REG_PIO3924EDGE);	//read, raise0 is falling
		mt7620_gpio_write(TP_MT7620_REG_PIO3924INT, 
						 (mt7620_gpio_intp | TP_MT7620_GPIO(mt7620_gpio_reg_offset(GPIO_RESET_FAC_CONTROL))));	//write 1 clear intp
		mt7620_gpio_write(TP_MT7620_REG_PIO3924EDGE, 
						 (mt7620_gpio_edge | TP_MT7620_GPIO(mt7620_gpio_reg_offset(GPIO_RESET_FAC_CONTROL))));	//write 1 clear 
	}
#endif

#ifdef GPIO_WPS_BIT_CONTROL
	#if (!defined(CONFIG_RALINK_GPIO_RE200)) && (!defined(CONFIG_RALINK_GPIO_RE210))
		if (GPIO_WPS_BIT_CONTROL != GPIO_RESET_FAC_CONTROL)
		{
			mt7620_gpio_wps_intp = mt7620_gpio_read(TP_MT7620_REG_PIOINT);	//read, whether the chanaged of interrupt
			mt7620_gpio_wps_edge = mt7620_gpio_read(TP_MT7620_REG_PIOEDGE);	//read, raise0 is falling
			mt7620_gpio_write(TP_MT7620_REG_PIOINT, 
							 (mt7620_gpio_wps_intp | TP_MT7620_GPIO(GPIO_WPS_BIT_CONTROL)));	//write 1 clear intp
			mt7620_gpio_write(TP_MT7620_REG_PIOEDGE, 
							 (mt7620_gpio_wps_edge | TP_MT7620_GPIO(GPIO_WPS_BIT_CONTROL)));	//write 1 clear 
		}else
	#endif	/* !CONFIG_RALINK_GPIO_RE200  */
		{
			mt7620_gpio_wps_intp = mt7620_gpio_intp;
			mt7620_gpio_wps_edge = mt7620_gpio_edge;
		}
#endif

#ifdef GPIO_POWER_ONOFF_CONTROL
		mt7620_gpio_power_onoff_intp = mt7620_gpio_read(TP_MT7620_REG_PIO3924INT);	//read, whether the chanaged of interrupt
		mt7620_gpio_power_onoff_edge = mt7620_gpio_read(TP_MT7620_REG_PIO3924EDGE);	//read, raise0 is falling
		mt7620_gpio_write(TP_MT7620_REG_PIO3924INT, 
						 (mt7620_gpio_power_onoff_intp | TP_MT7620_GPIO(mt7620_gpio_reg_offset(GPIO_POWER_ONOFF_CONTROL))));	//write 1 clear intp
		mt7620_gpio_write(TP_MT7620_REG_PIO3924EDGE, 
						 (mt7620_gpio_power_onoff_edge | TP_MT7620_GPIO(mt7620_gpio_reg_offset(GPIO_POWER_ONOFF_CONTROL))));	//write 1 clear 
#endif

#ifdef LED_POWER_SWITCH_ON_OFF
#if defined(CONFIG_RALINK_GPIO_RE210)
		mt7620_gpio_led_onoff_intp = mt7620_gpio_power_onoff_intp;
		mt7620_gpio_led_onoff_edge = mt7620_gpio_power_onoff_edge;
#endif
#endif
	return 0;
}

static int mt7620_gpio_enirq(void)
{
	u32 gpioirq;
	u32 gpiodir;

#ifdef 	GPIO_RESET_FAC_CONTROL
	if (GPIO_RESET_FAC_CONTROL <= 23)
	{
		/* config dir as input */
		gpiodir = mt7620_gpio_read(TP_MT7620_REG_PIODIR);
		gpiodir = mt7620_gpio_set_bit(GPIO_RESET_FAC_CONTROL,
								TP_MT7620_GPIO_VALUE_CLR,
								gpiodir);
		mt7620_gpio_write(TP_MT7620_REG_PIODIR, gpiodir);
		
		/* raise edge interrupt */
		gpioirq = mt7620_gpio_read(TP_MT7620_REG_PIORENA);
		gpioirq = mt7620_gpio_set_bit(GPIO_RESET_FAC_CONTROL,
									TP_MT7620_GPIO_VALUE_SET,
									gpioirq);
		mt7620_gpio_write(TP_MT7620_REG_PIORENA, gpioirq);

		/* falling edge interrupt */
		gpioirq = 0;
		gpioirq = mt7620_gpio_read(TP_MT7620_REG_PIOFENA);
		gpioirq = mt7620_gpio_set_bit(GPIO_RESET_FAC_CONTROL,
									TP_MT7620_GPIO_VALUE_SET,
									gpioirq);
		mt7620_gpio_write(TP_MT7620_REG_PIOFENA, gpioirq);

		
	}
	else if (GPIO_RESET_FAC_CONTROL <= 39 && GPIO_RESET_FAC_CONTROL >= 24)
	{
		/* config dir as input */
		gpiodir = mt7620_gpio_read(TP_MT7620_REG_PIO3924DIR);
		gpiodir = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_RESET_FAC_CONTROL),
								TP_MT7620_GPIO_VALUE_CLR,
								gpiodir);
		mt7620_gpio_write(TP_MT7620_REG_PIO3924DIR, gpiodir);
		
		/* raise edge interrupt */
		gpioirq = mt7620_gpio_read(TP_MT7620_REG_PIO3924RENA);
		gpioirq = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_RESET_FAC_CONTROL),
									TP_MT7620_GPIO_VALUE_SET,
									gpioirq);
		mt7620_gpio_write(TP_MT7620_REG_PIO3924RENA, gpioirq);

		/* falling edge interrupt */
		gpioirq = 0;
		gpioirq = mt7620_gpio_read(TP_MT7620_REG_PIO3924FENA);
		gpioirq = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_RESET_FAC_CONTROL),
									TP_MT7620_GPIO_VALUE_SET,
									gpioirq);
		mt7620_gpio_write(TP_MT7620_REG_PIO3924FENA, gpioirq);

	}
#endif

#ifdef GPIO_WPS_BIT_CONTROL
		/* config dir as input */
		gpiodir = mt7620_gpio_read(TP_MT7620_REG_PIODIR);
		gpiodir = mt7620_gpio_set_bit(GPIO_WPS_BIT_CONTROL,
						   TP_MT7620_GPIO_VALUE_CLR,
						   gpiodir);
		mt7620_gpio_write(TP_MT7620_REG_PIODIR, gpiodir);

		/* raise edge interrupt */
		gpioirq = mt7620_gpio_read(TP_MT7620_REG_PIORENA);
		gpioirq = mt7620_gpio_set_bit(GPIO_WPS_BIT_CONTROL,
									TP_MT7620_GPIO_VALUE_SET,
									gpioirq);
		mt7620_gpio_write(TP_MT7620_REG_PIORENA, gpioirq);

		/* falling edge interrupt */
		gpioirq = 0;
		gpioirq = mt7620_gpio_read(TP_MT7620_REG_PIOFENA);
		gpioirq = mt7620_gpio_set_bit(GPIO_WPS_BIT_CONTROL,
									TP_MT7620_GPIO_VALUE_SET,
									gpioirq);
		mt7620_gpio_write(TP_MT7620_REG_PIOFENA, gpioirq);
#endif

#ifdef GPIO_POWER_ONOFF_CONTROL
		/* config dir as input */
		gpiodir = mt7620_gpio_read(TP_MT7620_REG_PIO3924DIR);
		gpiodir = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_POWER_ONOFF_CONTROL),
						   GPIO_POWER_ONOFF_CONTROL_DIRVALUE,
						   gpiodir);
		mt7620_gpio_write(TP_MT7620_REG_PIO3924DIR, gpiodir);

		/* raise edge interrupt */
		gpioirq = mt7620_gpio_read(TP_MT7620_REG_PIO3924RENA);
		gpioirq = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_POWER_ONOFF_CONTROL),
									GPIO_POWER_ONOFF_CONTROL_RENAVALUE,
									gpioirq);
		mt7620_gpio_write(TP_MT7620_REG_PIO3924RENA, gpioirq);

		/* falling edge interrupt */
		gpioirq = 0;
		gpioirq = mt7620_gpio_read(TP_MT7620_REG_PIO3924FENA);
		gpioirq = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_POWER_ONOFF_CONTROL),
									GPIO_POWER_ONOFF_CONTROL_FENAVALUE,
									gpioirq);
		mt7620_gpio_write(TP_MT7620_REG_PIO3924FENA, gpioirq);
#endif

#if 1
#ifdef LED_POWER_SWITCH_ON_OFF
#if (!defined(CONFIG_RALINK_GPIO_RE200))
		/* config dir as input */
		gpiodir = mt7620_gpio_read(TP_MT7620_REG_PIO3924DIR);
		gpiodir = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_LED_POWER_SWITCH_BIT_CONTROL),
						   GPIO_LED_POWER_SWITCH_BIT_CONTROL_DIRVALUE,
						   gpiodir);
		mt7620_gpio_write(TP_MT7620_REG_PIO3924DIR, gpiodir);

		/* raise edge interrupt */
		gpioirq = mt7620_gpio_read(TP_MT7620_REG_PIO3924RENA);
		gpioirq = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_LED_POWER_SWITCH_BIT_CONTROL),
									GPIO_LED_POWER_SWITCH_BIT_CONTROL_RENAVALUE,
									gpioirq);
		mt7620_gpio_write(TP_MT7620_REG_PIO3924RENA, gpioirq);

		/* falling edge interrupt */
		gpioirq = 0;
		gpioirq = mt7620_gpio_read(TP_MT7620_REG_PIO3924FENA);
		gpioirq = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_LED_POWER_SWITCH_BIT_CONTROL),
									GPIO_LED_POWER_SWITCH_BIT_CONTROL_FENAVALUE,
									gpioirq);
		mt7620_gpio_write(TP_MT7620_REG_PIO3924FENA, gpioirq);
#endif        
#endif
#endif

		/* enable global gpio interrupt */
		gpioirq = mt7620_gpio_read(TP_MT7620_REG_INTENA);
		gpioirq = mt7620_gpio_set_bit(TP_MT7620_GLOBAL_IRQ,
									TP_MT7620_GPIO_VALUE_SET,
									gpioirq);
		mt7620_gpio_write(TP_MT7620_REG_INTENA, gpioirq);
	
	return 0;
}

static int mt7620_gpio_system_mode_read (char *page, char **start, off_t off,
               int count, int *eof, void *data)
{
	int system_mode;

	system_mode = g_sysMode;

	return sprintf(page, "%d\n", system_mode);
}

static int mt7620_gpio_system_mode_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	u_int32_t val = 0;
	
	if (sscanf(buf, "%d", &val) != 1)
	{
        return -EINVAL;
	}

	if (val < 0)
	{
		return -EINVAL;
	}
	
	if (val == SYSTEM_MODE_SOFT_ENABLE)	/* disable reading system mode from gpio pins */
	{
		g_isSoftSysModeEnable = 1;
	}
	else if (val == SYSTEM_MODE_SOFT_DISABLE)	/* enable reading system mode from gpio pins */
	{
		g_isSoftSysModeEnable = 0;
	}
	else if (g_isSoftSysModeEnable)
	{
		g_sysMode = val;	/* set system mode from web server */
	}

	return count;
}
#ifdef GPIO_ETHERNET_LED
static int mt7620_gpio_ethernet_led_read (char *page, char **start, off_t off,
                               int count, int *eof, void *data)
{
    return sprintf (page, "%d\n", g_ethernet_led);
}

static int mt7620_gpio_ethernet_led_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	u_int32_t val;
	u32 gpio_data;
	
	if (sscanf(buf, "%d", &val) != 1)
        return -EINVAL;

	if ((val < 0) || (val > 1))
	{
		return -EINVAL;
	}

	g_ethernet_led = val;

	if (g_ethernet_led)
	{		
		gpio_data = mt7620_gpio_read(TP_MT7620_REG_PIO7140DATA);
		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_ETHERNET_LED_BIT_CONTROL), 
												GPIO_ETHERNET_LED_BIT_CONTROL_ON, gpio_data);
		mt7620_gpio_write(TP_MT7620_REG_PIO7140DATA, gpio_data); 
	}
	else
	{
		gpio_data = mt7620_gpio_read(TP_MT7620_REG_PIO7140DATA);
		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_ETHERNET_LED_BIT_CONTROL), 
												GPIO_ETHERNET_LED_BIT_CONTROL_OFF, gpio_data);
		mt7620_gpio_write(TP_MT7620_REG_PIO7140DATA, gpio_data); 
	}

	return count;
}
#endif
#ifdef GPIO_WLAN_BACK_END_POST_LED
static int mt7620_gpio_wlan_back_end_led_read (char *page, char **start, off_t off,
                               int count, int *eof, void *data)
{
    return sprintf (page, "%d\n", g_wlan_back_end_led);
}

static int mt7620_gpio_wlan_back_end_led_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	u_int32_t val;
	u32 gpio_data;
	
	if (sscanf(buf, "%d", &val) != 1)
        return -EINVAL;

	if ((val < 0) || (val > 1))
	{
		return -EINVAL;
	}

	g_wlan_back_end_led = val;

	gpio_data = mt7620_gpio_read(TP_MT7620_REG_PIO3924DATA);
	if (g_wlan_back_end_led)
	{		
		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_WLAN_BACK_END_LED_BIT_CONTROL), 
											GPIO_WLAN_BACK_END_LED_BIT_CONTROL_ON, gpio_data);
	}
	else
	{
		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_WLAN_BACK_END_LED_BIT_CONTROL), 
											GPIO_WLAN_BACK_END_LED_BIT_CONTROL_OFF, gpio_data);
	}
	/*
	**	The on/off operation of wlan_back_end_led will cause range_extender_led on, 
	**	this is specially happened for re200v1(GPO39 used for range_extender_led). 
	**	So we will rewrite status of range_extender_led here.
	@@	by TengFei, 13/04/19.
	*/
	if (g_range_extender_led)
	{
		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_RANGE_EXTENDER_LED_BIT_CONTROL), 
		GPIO_RANGE_EXTENDER_LED_BIT_CONTROL_ON, gpio_data);
	}
	else
	{
		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_RANGE_EXTENDER_LED_BIT_CONTROL), 
		GPIO_RANGE_EXTENDER_LED_BIT_CONTROL_OFF, gpio_data);
	}
	mt7620_gpio_write(TP_MT7620_REG_PIO3924DATA, gpio_data); 

	return count;
}
#endif



#ifdef GPIO_RANGE_EXTENDER_LED
static int mt7620_gpio_range_extender_led_read (char *page, char **start, off_t off,
                               int count, int *eof, void *data)
{
    return sprintf (page, "%d\n", g_range_extender_led);
}

static int mt7620_gpio_range_extender_led_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	u_int32_t val;
	u32 gpio_data;
	
	if (sscanf(buf, "%d", &val) != 1)
        return -EINVAL;

	if ((val < 0) || (val > 1))
	{
		return -EINVAL;
	}

	g_range_extender_led = val;

	if (g_range_extender_led)
	{		
		gpio_data = mt7620_gpio_read(TP_MT7620_REG_PIO3924DATA);
		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_RANGE_EXTENDER_LED_BIT_CONTROL), 
		GPIO_RANGE_EXTENDER_LED_BIT_CONTROL_ON, gpio_data);
		mt7620_gpio_write(TP_MT7620_REG_PIO3924DATA, gpio_data); 
	}
	else
	{
		gpio_data = mt7620_gpio_read(TP_MT7620_REG_PIO3924DATA);
		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_RANGE_EXTENDER_LED_BIT_CONTROL), 
		GPIO_RANGE_EXTENDER_LED_BIT_CONTROL_OFF, gpio_data);
		mt7620_gpio_write(TP_MT7620_REG_PIO3924DATA, gpio_data); 
	}

	return count;
}
#endif
#ifdef GPIO_RANGE_EXTENDER_TRICOLOR_LED
static int mt7620_gpio_range_extender_tricolor_led_read (char *page, char **start, off_t off,
                               int count, int *eof, void *data)
{
    return sprintf (page, "%d\n", g_range_extender_tricolor_led);
}

static int mt7620_gpio_range_extender_tricolor_led_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	u_int32_t val;
	u32 gpio_data;
	u32 green_led_onoff = GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_OFF;
	u32 red_led_onoff = GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_OFF;
    
	typedef enum {
		RE_COLOR_NONE = 0,
		RE_COLOR_RED,
		RE_COLOR_GREEN,	
		RE_COLOR_MIX,		
	} WLAN_2G_LED_COLOR;
	
	if (sscanf(buf, "%d", &val) != 1)
        return -EINVAL;

	if ((val < 0) || (val > LED_STATE_MAX))
	{
		return -EINVAL;
	}
	if (val == g_range_extender_tricolor_led)
	{
	    return count;
	}

	g_range_extender_tricolor_led = val;

#ifdef LED_POWER_SWITCH_ON_OFF
#if defined(CONFIG_RALINK_GPIO_RE210)
    if (g_all_led_power_on_off == 0)
    {
        return count;
    }
#endif
#endif

	switch (val) 
	{
		case RE_COLOR_NONE:    /* none */
			green_led_onoff = GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_OFF;   
			red_led_onoff = GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_OFF;
			break;

		case RE_COLOR_RED:		/* red */
			green_led_onoff = GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_OFF;	
			red_led_onoff = GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_ON;
			break;

		case RE_COLOR_GREEN:	/* green */
			green_led_onoff = GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_ON;	
			red_led_onoff = GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_OFF;
			break;

		case RE_COLOR_MIX:		/* mix */
			green_led_onoff = GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_ON;		
			red_led_onoff = GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_ON;
			break;

		default:
			/* nothing to do. */
			;
	}
	
	gpio_data = mt7620_gpio_read(GPIO_RANGE_EXTENDER_TRICOLOR_LED_PREFIX(DATA));
	gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL0), 
					green_led_onoff, gpio_data);
	mt7620_gpio_write(GPIO_RANGE_EXTENDER_TRICOLOR_LED_PREFIX(DATA), gpio_data); 
	
	gpio_data = mt7620_gpio_read(GPIO_RANGE_EXTENDER_TRICOLOR_LED_PREFIX(DATA));
	gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL1), 
					red_led_onoff, gpio_data);
	mt7620_gpio_write(GPIO_RANGE_EXTENDER_TRICOLOR_LED_PREFIX(DATA), gpio_data); 
	
	return count;
}
#endif



#ifdef LED_POWER_SWITCH_ON_OFF
static int mt7620_gpio_all_led_power_on_off_read (char *page, char **start, off_t off,
                               int count, int *eof, void *data)
{
    return sprintf (page, "%d\n", g_all_led_power_on_off);
}

static int mt7620_gpio_all_led_power_on_off_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	u_int32_t val;
	u32 gpio_data;
	
	if (sscanf(buf, "%d", &val) != 1)
        return -EINVAL;

	if ((val < 0) || (val > 1))
	{
		return -EINVAL;
	}

	g_all_led_power_on_off = val;

#if defined(CONFIG_RALINK_GPIO_RE210)
    mt7620_gpio_set_all_led(g_all_led_power_on_off);
#else
	gpio_data = mt7620_gpio_read(GPIO_POWER_SWITCH_LED_PREFIX(DATA));
	if (g_all_led_power_on_off)
	{
		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_LED_POWER_SWITCH_BIT_CONTROL), 
		GPIO_LED_POWER_SWITCH_BIT_CONTROL_ON, gpio_data);
		/*
		**	The set operation of all_led_power_on_off will cause range_extender_led on, 
		**	this is specially happened for re200v1(GPO39 used for range_extender_led). 
		**	So we will rewrite status of range_extender_led here.
		@@	by TengFei, 13/04/19.
		*/
		if (g_range_extender_led)
		{
			gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_RANGE_EXTENDER_LED_BIT_CONTROL), 
			GPIO_RANGE_EXTENDER_LED_BIT_CONTROL_ON, gpio_data);
		}
		else
		{
			gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_RANGE_EXTENDER_LED_BIT_CONTROL), 
			GPIO_RANGE_EXTENDER_LED_BIT_CONTROL_OFF, gpio_data);
		}
		mt7620_gpio_write(TP_MT7620_REG_PIO3924DATA, gpio_data);       
	}
	else
	{
		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_LED_POWER_SWITCH_BIT_CONTROL), 
		GPIO_LED_POWER_SWITCH_BIT_CONTROL_OFF, gpio_data);
	}
	mt7620_gpio_write(GPIO_POWER_SWITCH_LED_PREFIX(DATA), gpio_data);     
#endif

	return count;
}
#endif

#ifdef WLAN_2G_GPIO_TRICOLOR_LED
static int mt7620_gpio_wlan_2g_tricolor_led_read (char *page, char **start, off_t off,
                               int count, int *eof, void *data)
{
    return sprintf (page, "%d\n", g_wlan_2g_tricolor_led);
}

static int mt7620_gpio_wlan_2g_tricolor_led_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	u_int32_t val;
	u32 gpio_data;
	u32 yellow_green_led_onoff = WLAN_2G_GPIO_TRICOLOR_LED_BIT_CONTROL_OFF;
	u32 red_led_onoff = WLAN_2G_GPIO_TRICOLOR_LED_BIT_CONTROL_OFF;
	typedef enum {
		WLAN_2G_COLOR_NONE = 0,
		WLAN_2G_COLOR_RED,
		WLAN_2G_COLOR_BLUE,	
		WLAN_2G_COLOR_YELLOW,		
	} WLAN_2G_LED_COLOR;
	
	if (sscanf(buf, "%d", &val) != 1)
        return -EINVAL;

	if ((val < 0) || (val > LED_STATE_MAX))
	{
		return -EINVAL;
	}
	if (val == g_wlan_2g_tricolor_led)
	{
	    return count;
	}

	g_wlan_2g_tricolor_led = val;

	switch (val) 
	{
		case WLAN_2G_COLOR_NONE:    /* none */
			yellow_green_led_onoff = WLAN_2G_GPIO_TRICOLOR_LED_BIT_CONTROL_OFF;   
			red_led_onoff = WLAN_2G_GPIO_TRICOLOR_LED_BIT_CONTROL_OFF;
			break;

		case WLAN_2G_COLOR_BLUE:		/* blue */
			yellow_green_led_onoff = WLAN_2G_GPIO_TRICOLOR_LED_BIT_CONTROL_ON;	
			red_led_onoff = WLAN_2G_GPIO_TRICOLOR_LED_BIT_CONTROL_OFF;
			break;

		case WLAN_2G_COLOR_RED:		/* red */
			yellow_green_led_onoff = WLAN_2G_GPIO_TRICOLOR_LED_BIT_CONTROL_OFF;	
			red_led_onoff = WLAN_2G_GPIO_TRICOLOR_LED_BIT_CONTROL_ON;
			break;

		case WLAN_2G_COLOR_YELLOW:		/* yellow */
			yellow_green_led_onoff = WLAN_2G_GPIO_TRICOLOR_LED_BIT_CONTROL_ON;		
			red_led_onoff = WLAN_2G_GPIO_TRICOLOR_LED_BIT_CONTROL_ON;
			break;

		default:
			/* nothing to do. */
			;
	}
	
	gpio_data = mt7620_gpio_read(TP_MT7620_REG_PIODATA);
	gpio_data = mt7620_gpio_set_bit(WLAN_2G_GPIO_TRICOLOR_LED_BIT_CONTROL0, 
					red_led_onoff, gpio_data);
	mt7620_gpio_write(TP_MT7620_REG_PIODATA, gpio_data); 
	
	gpio_data = mt7620_gpio_read(TP_MT7620_REG_PIO72DATA);
	gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(WLAN_2G_GPIO_TRICOLOR_LED_BIT_CONTROL1), 
					yellow_green_led_onoff, gpio_data);
	mt7620_gpio_write(TP_MT7620_REG_PIO72DATA, gpio_data); 
	
	return count;
}
#endif

#ifdef GPIO_2G_LED
static int mt7620_gpio_wlan_2g_led_read (char *page, char **start, off_t off,
                               int count, int *eof, void *data)
{
    return sprintf (page, "%d\n", g_wlan_2g_led);
}

static int mt7620_gpio_wlan_2g_led_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	u_int32_t val;
	u32 gpio_data;
	
	if (sscanf(buf, "%d", &val) != 1)
        return -EINVAL;

	if ((val < 0) || (val > 1))
	{
		return -EINVAL;
	}

	g_wlan_2g_led = val;

#ifdef LED_POWER_SWITCH_ON_OFF
#if defined(CONFIG_RALINK_GPIO_RE210)
    if (g_all_led_power_on_off == 0)
    {
        return count;
    }
#endif
#endif    

	gpio_data = mt7620_gpio_read(GPIO_2G_LED_PREFIX(DATA));
	if (g_wlan_2g_led)
	{		
		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_2G_LED_BIT_CONTROL), 
											GPIO_2G_LED_BIT_CONTROL_ON, gpio_data);
	}
	else
	{
		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_2G_LED_BIT_CONTROL), 
											GPIO_2G_LED_BIT_CONTROL_OFF, gpio_data);
	}

    mt7620_gpio_write(GPIO_2G_LED_PREFIX(DATA), gpio_data); 
	
	return count;
}
#endif

#ifdef GPIO_5G_LED
static int mt7620_gpio_wlan_5g_led_read (char *page, char **start, off_t off,
                               int count, int *eof, void *data)
{
    return sprintf (page, "%d\n", g_wlan_5g_led);
}

static int mt7620_gpio_wlan_5g_led_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	u_int32_t val;
	u32 gpio_data;
	
	if (sscanf(buf, "%d", &val) != 1)
        return -EINVAL;

	if ((val < 0) || (val > 1))
	{
		return -EINVAL;
	}

	g_wlan_5g_led = val;

#ifdef LED_POWER_SWITCH_ON_OFF
#if defined(CONFIG_RALINK_GPIO_RE210)
    if (g_all_led_power_on_off == 0)
    {
        return count;
    }
#endif
#endif    

	gpio_data = mt7620_gpio_read(GPIO_5G_LED_PREFIX(DATA));
	if (g_wlan_5g_led)
	{		
		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_5G_LED_BIT_CONTROL), 
											GPIO_5G_LED_BIT_CONTROL_ON, gpio_data);
	}
	else
	{
		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_5G_LED_BIT_CONTROL), 
											GPIO_5G_LED_BIT_CONTROL_OFF, gpio_data);
	}

    mt7620_gpio_write(GPIO_5G_LED_PREFIX(DATA), gpio_data); 
	
	return count;
}
#endif
#ifdef WLAN_5G_GPIO_TRICOLOR_LED
static int mt7620_gpio_wlan_5g_tricolor_led_read (char *page, char **start, off_t off,
                               int count, int *eof, void *data)
{
    return sprintf (page, "%d\n", g_wlan_5g_tricolor_led);
}

static int mt7620_gpio_wlan_5g_tricolor_led_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	u_int32_t val;
	u32 gpio_data;
	u32 green_led_onoff = WLAN_5G_GPIO_TRICOLOR_LED_BIT_CONTROL_OFF;
	u32 red_led_onoff = WLAN_5G_GPIO_TRICOLOR_LED_BIT_CONTROL_OFF;
	typedef enum {
		WLAN_5G_COLOR_NONE = 0,	
		WLAN_5G_COLOR_RED,
		WLAN_5G_COLOR_BLUE,
		WLAN_5G_COLOR_YELLOW,		
	} WLAN_5G_LED_COLOR;
	
	if (sscanf(buf, "%d", &val) != 1)
        return -EINVAL;

	if ((val < 0) || (val > LED_STATE_MAX))
	{
		return -EINVAL;
	}
	if (val == g_wlan_5g_tricolor_led)
	{
	    return count;
	}

	g_wlan_5g_tricolor_led = val;

	switch (val) 
	{
		case WLAN_5G_COLOR_NONE:    /* none */
			green_led_onoff = WLAN_5G_GPIO_TRICOLOR_LED_BIT_CONTROL_OFF;   
			red_led_onoff = WLAN_5G_GPIO_TRICOLOR_LED_BIT_CONTROL_OFF;
			break;

		case WLAN_5G_COLOR_BLUE:		/* blue */
			green_led_onoff = WLAN_5G_GPIO_TRICOLOR_LED_BIT_CONTROL_ON;	
			red_led_onoff = WLAN_5G_GPIO_TRICOLOR_LED_BIT_CONTROL_OFF;
			break;

		case WLAN_5G_COLOR_RED:		/* red */
			green_led_onoff = WLAN_5G_GPIO_TRICOLOR_LED_BIT_CONTROL_OFF;	
			red_led_onoff = WLAN_5G_GPIO_TRICOLOR_LED_BIT_CONTROL_ON;
			break;

		case WLAN_5G_COLOR_YELLOW:		/* yellow */
			green_led_onoff = WLAN_5G_GPIO_TRICOLOR_LED_BIT_CONTROL_ON;		
			red_led_onoff = WLAN_5G_GPIO_TRICOLOR_LED_BIT_CONTROL_ON;
			break;

		default:
			/* nothing to do. */
			;
	}
	
	gpio_data = mt7620_gpio_read(WLAN_5G_GPIO_TRICOLOR_ADDR);
	gpio_data = mt7620_gpio_set_bit(WLAN_5G_GPIO_TRICOLOR_LED_GREEN_BIT, 
					green_led_onoff, gpio_data);
	gpio_data = mt7620_gpio_set_bit(WLAN_5G_GPIO_TRICOLOR_LED_RED_BIT, 
					red_led_onoff, gpio_data);
	mt7620_gpio_write(WLAN_5G_GPIO_TRICOLOR_ADDR, gpio_data); 
	
	return count;
}
#endif



#ifdef GPIO_TRICOLOR_LED
static int mt7620_gpio_tricolor_led_read (char *page, char **start, off_t off,
               int count, int *eof, void *data)
{
    return sprintf (page, "%d\n", gpio_tricolorled);
}

static int mt7620_gpio_tricolor_led_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	u32 val;
	u32 gpio_data;
	u32 green_led_onoff = 0;
	u32 yellow_led_onoff = 0;
	
    if (sscanf(buf, "%d", &val) != 1)
    {
		return -EINVAL;
    }

	if (val >= LED_STATE_MAX)
	{
		return -EINVAL;
	}

    if (val == gpio_tricolorled)
    {
	    return count;
    }

	switch (val) 
	{
        case LED_STATE_OFF :
			green_led_onoff = TRICOLOR_LED_OFF;   /* both LEDs OFF */
			yellow_led_onoff = TRICOLOR_LED_OFF;
			break;

		case LED_STATE_GREEN:			/* no this status */
			green_led_onoff = TRICOLOR_LED_ON;		/* green ON, Yellow OFF */
			yellow_led_onoff = TRICOLOR_LED_OFF;
			break;

		case LED_STATE_YELLOW:		/* net connect, but have no data */
			green_led_onoff = TRICOLOR_LED_OFF;	/* green OFF, Yellow ON */
			yellow_led_onoff = TRICOLOR_LED_ON;
			break;

		case LED_STATE_ORANGE:		/* net not connect */
			green_led_onoff = TRICOLOR_LED_ON;		/* both LEDs ON */
			yellow_led_onoff = TRICOLOR_LED_ON;
			break;
	}

	printk("green_led_onoff = %d\n", green_led_onoff);

	gpio_data = mt7620_gpio_read(TP_MT7620_REG_PIODATA);
	gpio_data = mt7620_gpio_set_bit(GPIO_TRICOLOR_LED_CONTROL0, green_led_onoff, gpio_data);
	gpio_data = mt7620_gpio_set_bit(GPIO_TRICOLOR_LED_CONTROL1, yellow_led_onoff, gpio_data);
	mt7620_gpio_write(TP_MT7620_REG_PIODATA, gpio_data);

    gpio_tricolorled = val;

    return count;
} 
#endif

/* add by xiongliuzhong, 01Apr13, we should modify this func as the function */
#ifdef GPIO_INTERNET_LED
void mt7620_gpio_turn_on_internet_led(void)
{
	u32 gpio_data;
	
	gpio_data = mt7620_gpio_read(TP_MT7620_REG_PIODATA);
	gpio_data = mt7620_gpio_set_bit(GPIO_INTERNET_LED_CONTROL0, INTERNET_LED_ON, gpio_data);
	mt7620_gpio_write(TP_MT7620_REG_PIODATA, gpio_data);
}
EXPORT_SYMBOL(mt7620_gpio_turn_on_internet_led);

void mt7620_gpio_turn_off_internet_led(void)
{
	u32 gpio_data;
	
	gpio_data = mt7620_gpio_read(TP_MT7620_REG_PIODATA);
	gpio_data = mt7620_gpio_set_bit(GPIO_INTERNET_LED_CONTROL0, INTERNET_LED_OFF, gpio_data);
	mt7620_gpio_write(TP_MT7620_REG_PIODATA, gpio_data);
}
EXPORT_SYMBOL(mt7620_gpio_turn_off_internet_led);

static int mt7620_gpio_internet_status_read (char *page, char **start, off_t off,
                               int count, int *eof, void *data)
{
    return sprintf (page, "%d\n", g_internetLedStatus);
}

/*
* buf: 0 means not connected to internet; 1 means 3G connected; 2 means wisp connected; 3 means wired connected.
*/
static int mt7620_gpio_internet_status_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
    u_int32_t val;

	if (sscanf(buf, "%d", &val) != 1)
        return -EINVAL;

	if ((val < 0) || (val > 3))
	{
		return -EINVAL;
	}

	g_internetLedStatus = val;

	if (g_internetLedStatus)
	{
		mt7620_gpio_turn_on_internet_led();
	}
	else
	{
		mt7620_gpio_turn_off_internet_led();
	}

	return count;
}
#endif

#ifdef GPIO_POWER_SYSTEM_LED
static int mt7620_gpio_power_system_led_read (char *page, char **start, off_t off,
                               int count, int *eof, void *data)
{
    return sprintf (page, "%d\n", g_power_system_led);
}

static int mt7620_gpio_power_system_led_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	u_int32_t val;
	u32 gpio_data;
	
	if (sscanf(buf, "%d", &val) != 1)
        return -EINVAL;

	if ((val < 0) || (val > 1))
	{
		return -EINVAL;
	}

	g_power_system_led = val;

#ifdef LED_POWER_SWITCH_ON_OFF
#if defined(CONFIG_RALINK_GPIO_RE210)
    if (g_all_led_power_on_off == 0)
    {
        return count;
    }
#endif
#endif    

	gpio_data = mt7620_gpio_read(GPIO_POWER_SYSTEM_LED_PREFIX(DATA));
	if (g_power_system_led)
	{		
		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_POWER_SYSTEM_LED_BIT_CONTROL), 
											GPIO_POWER_SYSTEM_LED_BIT_CONTROL_ON, gpio_data);
	}
	else
	{
		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_POWER_SYSTEM_LED_BIT_CONTROL), 
											GPIO_POWER_SYSTEM_LED_BIT_CONTROL_OFF, gpio_data);
	}

    mt7620_gpio_write(GPIO_POWER_SYSTEM_LED_PREFIX(DATA), gpio_data); 
	
	return count;
}


static int mt7620_gpio_power_system_led_blink_read (char *page, char **start, off_t off,
                               int count, int *eof, void *data)
{
    return sprintf (page, "%d\n", system_led_blink);
}

static int mt7620_gpio_power_system_led_blink_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	u_int32_t val;
	u32 gpio_data;
	
	if (sscanf(buf, "%d", &val) != 1)
        return -EINVAL;

	if ((val < 0) || (val > 1))
	{
		return -EINVAL;
	}

	system_led_blink = val;
	
    del_timer(&mt7620_gpio_power_blink_timer);
	mt7620_gpio_power_blink_timer.expires = jiffies + HZ/2;
	mt7620_gpio_power_blink_timer.function = mt7620_gpio_check_power_blink;
	add_timer(&mt7620_gpio_power_blink_timer);
	
	return count;
}
#endif

#ifdef GPIO_POWER_ONOFF_CONTROL
static int mt7620_gpio_power_onoff_status_read (char *page, char **start, off_t off,
                               int count, int *eof, void *data)
{
    return sprintf (page, "%d\n", mt7620_gpio_power_onoff_status);
}

static int mt7620_gpio_power_onoff_status_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	u_int32_t val;
	u32 gpio_data;
	
	if (sscanf(buf, "%d", &val) != 1)
        return -EINVAL;

	if ((val < 0) || (val > 1))
	{
		return -EINVAL;
	}

	mt7620_gpio_power_onoff_status = val;
	
	return count;
}
#endif


int gpio_power_system_led_thread(void *__unused)
{
#ifdef GPIO_POWER_SYSTEM_LED    
    int poll_interval = 500;
    u32 gpio_data;

    for(;;)
    {
		msleep_interruptible(poll_interval);
		if (kthread_should_stop())
			break;

        if (system_led_blink)
        {
        	g_power_system_led = (!g_power_system_led);

#ifdef LED_POWER_SWITCH_ON_OFF
#if defined(CONFIG_RALINK_GPIO_RE210)
            if (g_all_led_power_on_off == 0)
            {
                continue;
            }
#endif
#endif            
        	gpio_data = mt7620_gpio_read(GPIO_POWER_SYSTEM_LED_PREFIX(DATA));
        	if (g_power_system_led)
        	{		
        		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_POWER_SYSTEM_LED_BIT_CONTROL), 
        											GPIO_POWER_SYSTEM_LED_BIT_CONTROL_ON, gpio_data);
        	}
        	else
        	{
        		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_POWER_SYSTEM_LED_BIT_CONTROL), 
        											GPIO_POWER_SYSTEM_LED_BIT_CONTROL_OFF, gpio_data);
        	}

            mt7620_gpio_write(GPIO_POWER_SYSTEM_LED_PREFIX(DATA), gpio_data); 
        }
    }
#endif    
	return 0;
}

static void mt7620_gpio_check_power_blink(unsigned long nothing)
{
	u32 gpio_data;

	//printk("System_led_blink: %d, powerOnOff: %d\n", system_led_blink, g_all_led_power_on_off);
#ifdef GPIO_POWER_SYSTEM_LED 
		if (system_led_blink)
        {
            del_timer(&mt7620_gpio_power_blink_timer);
			mt7620_gpio_power_blink_timer.expires = jiffies + HZ/2;
			mt7620_gpio_power_blink_timer.function = mt7620_gpio_check_power_blink;
			add_timer(&mt7620_gpio_power_blink_timer);
			
#ifdef LED_POWER_SWITCH_ON_OFF
#if defined(CONFIG_RALINK_GPIO_RE210)
            if (g_all_led_power_on_off == 0)
            {
				return;
            }
#endif
#endif     			
        	g_power_system_led = (!g_power_system_led);
        	gpio_data = mt7620_gpio_read(GPIO_POWER_SYSTEM_LED_PREFIX(DATA));
        	if (g_power_system_led)
        	{		
        		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_POWER_SYSTEM_LED_BIT_CONTROL), 
        											GPIO_POWER_SYSTEM_LED_BIT_CONTROL_ON, gpio_data);
        	}
        	else
        	{
        		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_POWER_SYSTEM_LED_BIT_CONTROL), 
        											GPIO_POWER_SYSTEM_LED_BIT_CONTROL_OFF, gpio_data);
        	}

            mt7620_gpio_write(GPIO_POWER_SYSTEM_LED_PREFIX(DATA), gpio_data);
        }
		else
		{
			del_timer(&mt7620_gpio_power_blink_timer);
		}
#endif    
	return ;
}


#ifdef CONFIG_RALINK_MT7620
static int mt7620_gpio_wps_process_pid_read (char *page, char **start, off_t off,
                               int count, int *eof, void *data)
{
    return sprintf (page, "%d\n", g_wpsProcessPid);
}

static int mt7620_gpio_wps_process_pid_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
    u_int32_t val;

	if (sscanf(buf, "%d", &val) != 1)
        return -EINVAL;

	if (val < 0)
	{
		return -EINVAL;
	}

	g_wpsProcessPid = val;

	return count;
}
#endif


/*create proc file*/
static int mt7620_gpio_create_simple_config_proc(void)
{
	if (simple_config_entry != NULL)
	{
		printk ("Already have a proc entry for /proc/simple_config!\n");
		return -ENOENT;
	}

	simple_config_entry = proc_mkdir("simple_config", NULL);
	if (!simple_config_entry)
	{
		return -ENOENT;
	}

	/* for ralink_platform wps process */
#ifdef CONFIG_RALINK_MT7620
	wps_process_pid_entry = create_proc_entry ("wps_process_pid", 0666,
                                                      simple_config_entry);
	if (!wps_process_pid_entry)
	{
		return -ENOENT;
	}
	
    wps_process_pid_entry->write_proc = mt7620_gpio_wps_process_pid_write;
    wps_process_pid_entry->read_proc = mt7620_gpio_wps_process_pid_read;
#endif

	system_mode_entry = create_proc_entry("system_mode", 0666,
								simple_config_entry);
	if (!system_mode_entry)
	{
		return -ENOENT;
	}

	system_mode_entry->read_proc = mt7620_gpio_system_mode_read;
	system_mode_entry->write_proc = mt7620_gpio_system_mode_write; 

#ifdef GPIO_ETHERNET_LED
	ethernet_led_entry = create_proc_entry("ethernet_led", 0666, simple_config_entry);
	if (!ethernet_led_entry)
	{
		return -ENOENT;
	}

	ethernet_led_entry->read_proc = mt7620_gpio_ethernet_led_read;
	ethernet_led_entry->write_proc = mt7620_gpio_ethernet_led_write; 
#endif

#ifdef GPIO_RANGE_EXTENDER_LED
	range_extender_led_entry = create_proc_entry("range_extender_led", 0666, simple_config_entry);
	if (!range_extender_led_entry)
	{
		return -ENOENT;
	}

	range_extender_led_entry->read_proc = mt7620_gpio_range_extender_led_read;
	range_extender_led_entry->write_proc = mt7620_gpio_range_extender_led_write; 	
#endif
#ifdef GPIO_RANGE_EXTENDER_TRICOLOR_LED
	range_extender_tricolor_led_entry = create_proc_entry("range_extender_tricolor_led", 0666, simple_config_entry);
	if (!range_extender_tricolor_led_entry)
	{
		return -ENOENT;
	}

	range_extender_tricolor_led_entry->read_proc = mt7620_gpio_range_extender_tricolor_led_read;
	range_extender_tricolor_led_entry->write_proc = mt7620_gpio_range_extender_tricolor_led_write; 	
#endif


#ifdef GPIO_2G_LED
	wlan_2g_led_entry = create_proc_entry("wlan_2g_led", 0666, simple_config_entry);
	if (!wlan_2g_led_entry)
	{
		return -ENOENT;
	}

	wlan_2g_led_entry->read_proc = mt7620_gpio_wlan_2g_led_read;
	wlan_2g_led_entry->write_proc = mt7620_gpio_wlan_2g_led_write; 
#endif
#ifdef WLAN_2G_GPIO_TRICOLOR_LED
	wlan_2g_tricolor_led_entry = create_proc_entry("wlan_2g_tricolor_led", 0666, simple_config_entry);
	if (!wlan_2g_tricolor_led_entry)
	{
		return -ENOENT;
	}

	wlan_2g_tricolor_led_entry->read_proc = mt7620_gpio_wlan_2g_tricolor_led_read;
	wlan_2g_tricolor_led_entry->write_proc = mt7620_gpio_wlan_2g_tricolor_led_write; 
#endif

#ifdef GPIO_5G_LED
	wlan_5g_led_entry = create_proc_entry("wlan_5g_led", 0666, simple_config_entry);
	if (!wlan_5g_led_entry)
	{
		return -ENOENT;
	}

	wlan_5g_led_entry->read_proc = mt7620_gpio_wlan_5g_led_read;
	wlan_5g_led_entry->write_proc = mt7620_gpio_wlan_5g_led_write; 
#endif
#ifdef WLAN_5G_GPIO_TRICOLOR_LED
	wlan_5g_tricolor_led_entry = create_proc_entry("wlan_5g_tricolor_led", 0666, simple_config_entry);
	if (!wlan_5g_tricolor_led_entry)
	{
		return -ENOENT;
	}

	wlan_5g_tricolor_led_entry->read_proc = mt7620_gpio_wlan_5g_tricolor_led_read;
	wlan_5g_tricolor_led_entry->write_proc = mt7620_gpio_wlan_5g_tricolor_led_write; 
#endif	/* WLAN_5G_GPIO_TRICOLOR_LED */

#ifdef GPIO_WLAN_BACK_END_POST_LED
	wlan_back_end_led_entry = create_proc_entry("wlan_back_end_led", 0666, simple_config_entry);
	if (!wlan_back_end_led_entry)
	{
		return -ENOENT;
	}

	wlan_back_end_led_entry->read_proc = mt7620_gpio_wlan_back_end_led_read;
	wlan_back_end_led_entry->write_proc = mt7620_gpio_wlan_back_end_led_write; 
#endif

#ifdef LED_POWER_SWITCH_ON_OFF
	all_led_power_on_off_entry = create_proc_entry("all_led_power_on_off", 0666, simple_config_entry);
	if (!all_led_power_on_off_entry)
	{
		return -ENOENT;
	}

	all_led_power_on_off_entry->read_proc = mt7620_gpio_all_led_power_on_off_read;
	all_led_power_on_off_entry->write_proc = mt7620_gpio_all_led_power_on_off_write; 
#endif

#ifdef GPIO_POWER_SYSTEM_LED
	power_system_led_entry = create_proc_entry("power_system_led", 0666, simple_config_entry);
	if (!power_system_led_entry)
	{
		return -ENOENT;
	}

	power_system_led_entry->read_proc = mt7620_gpio_power_system_led_read;
	power_system_led_entry->write_proc = mt7620_gpio_power_system_led_write;

	power_system_led_blink_entry = create_proc_entry("power_system_led_blink", 0666, simple_config_entry);
	if (!power_system_led_blink_entry)
	{
		return -ENOENT;
	}

	power_system_led_blink_entry->read_proc = mt7620_gpio_power_system_led_blink_read;
	power_system_led_blink_entry->write_proc = mt7620_gpio_power_system_led_blink_write;    
#endif

#ifdef GPIO_POWER_ONOFF_CONTROL
	power_onoff_entry = create_proc_entry("power_onoff_status", 0666, simple_config_entry);
	if (!power_onoff_entry)
	{
		return -ENOENT;
	}

	power_onoff_entry->read_proc = mt7620_gpio_power_onoff_status_read;
	power_onoff_entry->write_proc = mt7620_gpio_power_onoff_status_write;   
#endif
	
	/*wlan 2g led off when linux startup. by hlf 20130730*/
#ifdef WLAN_2G_GPIO_TRICOLOR_LED
	g_wlan_2g_tricolor_led = 1;
	mt7620_gpio_wlan_2g_tricolor_led_write(NULL, "0", 0, NULL);
#endif
	return 0;
}

void do_send_wps_signal_to_wps_process_id(void)
{
	struct task_struct *wps_pthread_pid = NULL;
	
	if (!mt7620_gpio_block_wps)
	{
		if(-1 == g_wpsProcessPid)
		{
			printk("kernel, wps pthread not prepare OK!\n");
			return;
		}
		
		#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,35)
			wps_pthread_pid = find_task_by_vpid(g_wpsProcessPid);
		#else
			wps_pthread_pid = find_task_by_pid(g_wpsProcessPid);
		#endif

		if (NULL == wps_pthread_pid) 
		{
			printk(KERN_ERR NAME ":kernel, no registered process to notify\n");
			return;
		}

		printk(KERN_NOTICE NAME ":kernel, sending a SIGUSR1 to process %d\n",
				g_wpsProcessPid);
		send_sig(SIGUSR1, wps_pthread_pid, 0);
	}

}

static void mt7620_gpio_check_rst(unsigned long nothing)
{

	u32 gpio_data;

#ifdef	RST_HOLD_TIME	
	if (mt7620_gpio_rst_time == (RST_HOLD_TIME - 1))
	{
		printk("hold reset button 5s.\n");
#ifndef GPIO_WPS_BIT_CONTROL
		mt7620_gpio_block_wps    = 1;
#endif
		del_timer(&mt7620_gpio_rst_timer);
		mt7620_gpio_rst_time = 0;
		
 #ifdef  CONFIG_TP_GPIO_EVENT
		gpio_event_gset(TP_GPIO_SET_EVT, TP_GPIO_EVT_RST, 1);
 #else
 		mt7620_gpio_reset_enable = 1;
#endif  /* CONFIG_TP_GPIO_EVENT */
		return;
	}
#endif	/* RST_HOLD_TIME */

	/*get the data of gpio*/
	if (GPIO_RESET_FAC_CONTROL <= 23)
	{
		gpio_data = mt7620_gpio_read(TP_MT7620_REG_PIODATA);
	}
	else if (GPIO_RESET_FAC_CONTROL <= 39 && GPIO_RESET_FAC_CONTROL >= 24)
	{
		gpio_data = mt7620_gpio_read(TP_MT7620_REG_PIO3924DATA);
	}
	
	if (!mt7620_gpio_is_set(mt7620_gpio_reg_offset(GPIO_RESET_FAC_CONTROL), gpio_data)
#ifdef GPIO_POWER_ONOFF_CONTROL
		&&(mt7620_gpio_power_onoff_status)					
#endif
	)
	{
	#ifdef	RST_HOLD_TIME
		mt7620_gpio_rst_time++;
		printk("\n\t\t\t>>kernelw3294<< [%s] %d,mt7620_gpio_rst_time:%d \n", __FUNCTION__, __LINE__,mt7620_gpio_rst_time);
		mod_timer(&mt7620_gpio_rst_timer, jiffies + 1 * HZ);
	#else	/* !RST_HOLD_TIME */

		#ifndef GPIO_WPS_BIT_CONTROL
			mt7620_gpio_block_wps    = 1;
		#endif
		printk("RESET button has been pushed.\n");
		#ifdef  CONFIG_TP_GPIO_EVENT
			gpio_event_gset(TP_GPIO_SET_EVT, TP_GPIO_EVT_RST, 1);
 		#else
 			mt7620_gpio_reset_enable = 1;
		#endif  /* CONFIG_TP_GPIO_EVENT */
	#endif	/* RST_HOLD_TIME */
	}

#ifdef	RST_HOLD_TIME
	else
	{
		del_timer(&mt7620_gpio_rst_timer);
		mt7620_gpio_rst_time = 0;
#ifndef GPIO_WPS_BIT_CONTROL		
		mt7620_gpio_block_wps = 0;
#endif
	}
	
#ifndef GPIO_WPS_BIT_CONTROL
	do_send_wps_signal_to_wps_process_id();
#endif

#else	/* !RST_HOLD_TIME */
	del_timer(&mt7620_gpio_rst_timer);
	mt7620_gpio_rst_time = 0;
#endif	/* RST_HOLD_TIME */

	return;
}
#ifdef GPIO_WPS_BIT_CONTROL
static void mt7620_gpio_check_wps(unsigned long nothing)
{
	u32 gpio_data;
	
	/*get the data of gpio*/
	if (GPIO_WPS_BIT_CONTROL <= 23)
	{
		gpio_data = mt7620_gpio_read(TP_MT7620_REG_PIODATA);
	}
	else if (GPIO_WPS_BIT_CONTROL <= 39 && GPIO_WPS_BIT_CONTROL >= 24)
	{
		gpio_data = mt7620_gpio_read(TP_MT7620_REG_PIO3924DATA);
	}
	
	if (!mt7620_gpio_is_set(mt7620_gpio_reg_offset(GPIO_WPS_BIT_CONTROL), gpio_data)
#ifdef GPIO_POWER_ONOFF_CONTROL
		&&(mt7620_gpio_power_onoff_status)					
#endif
	)
	{
		printk("wps button has been pushed.\n");
		do_send_wps_signal_to_wps_process_id();
	}
	
	del_timer(&mt7620_gpio_wps_timer);
	mt7620_gpio_block_wps = 1;
}
#endif

#ifdef GPIO_POWER_ONOFF_CONTROL
static void mt7620_gpio_check_power_onoff(unsigned long nothing)
{
	u32 gpio_data;
	
	/*get the data of gpio*/
	if (GPIO_POWER_ONOFF_CONTROL <= 39 && GPIO_POWER_ONOFF_CONTROL >= 24)
	{
		gpio_data = mt7620_gpio_read(TP_MT7620_REG_PIO3924DATA);
	}
	
	if (!mt7620_gpio_is_set(mt7620_gpio_reg_offset(GPIO_POWER_ONOFF_CONTROL), gpio_data))
	{
		printk("power onoff button has been pushed.\n");
        mt7620_gpio_power_onoff_status = (!mt7620_gpio_power_onoff_status);
	}
	
	del_timer(&mt7620_gpio_power_onoff_timer);	
}
#endif
#ifdef LED_POWER_SWITCH_ON_OFF
#if defined(CONFIG_RALINK_GPIO_RE210)
static void mt7620_gpio_set_all_led(int onoff)
{
    u32 gpio_data;
    if (onoff)
    {
#ifdef GPIO_POWER_SYSTEM_LED 
    	gpio_data = mt7620_gpio_read(GPIO_POWER_SYSTEM_LED_PREFIX(DATA));
    	if (g_power_system_led)
    	{		
    		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_POWER_SYSTEM_LED_BIT_CONTROL), 
    											GPIO_POWER_SYSTEM_LED_BIT_CONTROL_ON, gpio_data);
    	}
    	else
    	{
    		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_POWER_SYSTEM_LED_BIT_CONTROL), 
    											GPIO_POWER_SYSTEM_LED_BIT_CONTROL_OFF, gpio_data);
    	}
        mt7620_gpio_write(GPIO_POWER_SYSTEM_LED_PREFIX(DATA), gpio_data); 
#endif
#ifdef GPIO_2G_LED
    	gpio_data = mt7620_gpio_read(GPIO_2G_LED_PREFIX(DATA));
    	if (g_wlan_2g_led)
    	{		
    		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_2G_LED_BIT_CONTROL), 
    											GPIO_2G_LED_BIT_CONTROL_ON, gpio_data);
    	}
    	else
    	{
    		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_2G_LED_BIT_CONTROL), 
    											GPIO_2G_LED_BIT_CONTROL_OFF, gpio_data);
    	}
        mt7620_gpio_write(GPIO_2G_LED_PREFIX(DATA), gpio_data); 
#endif
#ifdef GPIO_5G_LED
    	gpio_data = mt7620_gpio_read(GPIO_5G_LED_PREFIX(DATA));
    	if (g_wlan_5g_led)
    	{		
    		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_5G_LED_BIT_CONTROL), 
    											GPIO_5G_LED_BIT_CONTROL_ON, gpio_data);
    	}
    	else
    	{
    		gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_5G_LED_BIT_CONTROL), 
    											GPIO_5G_LED_BIT_CONTROL_OFF, gpio_data);
    	}
        mt7620_gpio_write(GPIO_5G_LED_PREFIX(DATA), gpio_data); 
#endif
#ifdef GPIO_RANGE_EXTENDER_TRICOLOR_LED
    	u32 green_led_onoff = GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_OFF;
    	u32 red_led_onoff = GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_OFF;
        
    	typedef enum {
    		RE_COLOR_NONE = 0,
    		RE_COLOR_RED,
    		RE_COLOR_GREEN,	
    		RE_COLOR_MIX,		
    	} WLAN_2G_LED_COLOR;

    	switch (g_range_extender_tricolor_led) 
    	{
    		case RE_COLOR_NONE:    /* none */
    			green_led_onoff = GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_OFF;   
    			red_led_onoff = GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_OFF;
    			break;

    		case RE_COLOR_RED:		/* red */
    			green_led_onoff = GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_OFF;	
    			red_led_onoff = GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_ON;
    			break;

    		case RE_COLOR_GREEN:	/* green */
    			green_led_onoff = GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_ON;	
    			red_led_onoff = GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_OFF;
    			break;

    		case RE_COLOR_MIX:		/* mix */
    			green_led_onoff = GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_ON;		
    			red_led_onoff = GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_ON;
    			break;

    		default:
    			/* nothing to do. */
    			;
    	}
    	
    	gpio_data = mt7620_gpio_read(GPIO_RANGE_EXTENDER_TRICOLOR_LED_PREFIX(DATA));
    	gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL0), 
    					green_led_onoff, gpio_data);
    	mt7620_gpio_write(GPIO_RANGE_EXTENDER_TRICOLOR_LED_PREFIX(DATA), gpio_data); 
    	
    	gpio_data = mt7620_gpio_read(GPIO_RANGE_EXTENDER_TRICOLOR_LED_PREFIX(DATA));
    	gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL1), 
    					red_led_onoff, gpio_data);
    	mt7620_gpio_write(GPIO_RANGE_EXTENDER_TRICOLOR_LED_PREFIX(DATA), gpio_data);   
#endif    
    }
    else
    {
#ifdef GPIO_POWER_SYSTEM_LED 
    	gpio_data = mt7620_gpio_read(GPIO_POWER_SYSTEM_LED_PREFIX(DATA));
    	gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_POWER_SYSTEM_LED_BIT_CONTROL), 
    										GPIO_POWER_SYSTEM_LED_BIT_CONTROL_OFF, gpio_data);
        mt7620_gpio_write(GPIO_POWER_SYSTEM_LED_PREFIX(DATA), gpio_data);
#endif
#ifdef GPIO_2G_LED
    	gpio_data = mt7620_gpio_read(GPIO_2G_LED_PREFIX(DATA));
    	gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_2G_LED_BIT_CONTROL), 
    										GPIO_2G_LED_BIT_CONTROL_OFF, gpio_data);
        mt7620_gpio_write(GPIO_2G_LED_PREFIX(DATA), gpio_data);
#endif
#ifdef GPIO_5G_LED
    	gpio_data = mt7620_gpio_read(GPIO_5G_LED_PREFIX(DATA));
    	gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_5G_LED_BIT_CONTROL), 
    										GPIO_5G_LED_BIT_CONTROL_OFF, gpio_data);
        mt7620_gpio_write(GPIO_5G_LED_PREFIX(DATA), gpio_data);
#endif
#ifdef GPIO_RANGE_EXTENDER_TRICOLOR_LED
    	gpio_data = mt7620_gpio_read(GPIO_RANGE_EXTENDER_TRICOLOR_LED_PREFIX(DATA));
    	gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL0), 
    					GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_OFF, gpio_data);
    	mt7620_gpio_write(GPIO_RANGE_EXTENDER_TRICOLOR_LED_PREFIX(DATA), gpio_data); 
    	
    	gpio_data = mt7620_gpio_read(GPIO_RANGE_EXTENDER_TRICOLOR_LED_PREFIX(DATA));
    	gpio_data = mt7620_gpio_set_bit(mt7620_gpio_reg_offset(GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL1), 
    					GPIO_RANGE_EXTENDER_TRICOLOR_LED_BIT_CONTROL_OFF, gpio_data);
    	mt7620_gpio_write(GPIO_RANGE_EXTENDER_TRICOLOR_LED_PREFIX(DATA), gpio_data);        
#endif          
    }      
}
static void mt7620_gpio_check_led_onoff(unsigned long nothing)
{
	u32 gpio_data;
	
	/*get the data of gpio*/
	if (GPIO_LED_POWER_SWITCH_BIT_CONTROL <= 39 && GPIO_LED_POWER_SWITCH_BIT_CONTROL >= 24)
	{
		gpio_data = mt7620_gpio_read(TP_MT7620_REG_PIO3924DATA);
	}
	
	if (!mt7620_gpio_is_set(mt7620_gpio_reg_offset(GPIO_LED_POWER_SWITCH_BIT_CONTROL), gpio_data)
#ifdef GPIO_POWER_ONOFF_CONTROL
		&&(mt7620_gpio_power_onoff_status) 				
#endif
	)
	{
		printk("led onoff button has been pushed.\n");
        g_all_led_power_on_off = (!g_all_led_power_on_off);
        mt7620_gpio_set_all_led(g_all_led_power_on_off);
	}
	
	del_timer(&mt7620_gpio_led_onoff_timer);	
}
#endif
#endif

static int mt7620_gpio_common_init(void)
{
	mt7620_gpio_setmode();
	mt7620_gpio_setdir();
	//mt7620_gpio_setpol();
	/* 
	**	we needn't to set status of leds, this has been done in uboot stage.
	@@	By TengFei.
	*/
#if	0
	mt7620_gpio_setdata();
#endif

	mt7620_gpio_enirq();

    return 0;
}

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
void mt7620_gpio_intp_gpio_irq_handler(unsigned int irq, struct irqaction *irqaction)
#else
irqreturn_t mt7620_gpio_intp_gpio_irq_handler(int irq, void *irqaction)
#endif
{
	mt7620_gpio_save_clear_intp();

	if (mt7620_gpio_is_set(mt7620_gpio_reg_offset(GPIO_RESET_FAC_CONTROL), mt7620_gpio_intp) 
		&& TP_MT7620_GPIO_FALLING_EDGE == mt7620_gpio_is_set(mt7620_gpio_reg_offset(GPIO_RESET_FAC_CONTROL), mt7620_gpio_edge))
	{
		if (0 == mt7620_gpio_factory_reset_inited)
		{
			return IRQ_HANDLED;
		}

		mt7620_gpio_rst_time = 0;
#ifndef GPIO_WPS_BIT_CONTROL		
		mt7620_gpio_block_wps = 1;
#endif

	#ifdef	RST_HOLD_TIME
		mod_timer(&mt7620_gpio_rst_timer, jiffies + 1 * HZ);
	#else	/* !RST_HOLD_TIME */
		mod_timer(&mt7620_gpio_rst_timer, jiffies + HZ/10);
	#endif	/* RST_HOLD_TIME */
	}

#ifdef GPIO_WPS_BIT_CONTROL
	if (mt7620_gpio_is_set(GPIO_WPS_BIT_CONTROL, mt7620_gpio_wps_intp) 
		&& TP_MT7620_GPIO_FALLING_EDGE == mt7620_gpio_is_set(GPIO_WPS_BIT_CONTROL, mt7620_gpio_wps_edge))
	{
		if (0 == mt7620_gpio_factory_reset_inited)
		{
			return IRQ_HANDLED;
		}
		mt7620_gpio_block_wps = 0;
		mod_timer(&mt7620_gpio_wps_timer, jiffies + HZ/10);
	}
#endif

#ifdef GPIO_POWER_ONOFF_CONTROL
	if (mt7620_gpio_is_set(mt7620_gpio_reg_offset(GPIO_POWER_ONOFF_CONTROL), mt7620_gpio_power_onoff_intp) 
		&& TP_MT7620_GPIO_FALLING_EDGE == 
		mt7620_gpio_is_set(mt7620_gpio_reg_offset(GPIO_POWER_ONOFF_CONTROL), mt7620_gpio_power_onoff_edge))
	{
		if (0 == mt7620_gpio_factory_reset_inited)
		{
			return IRQ_HANDLED;
		}
		mod_timer(&mt7620_gpio_power_onoff_timer, jiffies + HZ/10);
	}
#endif

#ifdef LED_POWER_SWITCH_ON_OFF
#if defined(CONFIG_RALINK_GPIO_RE210)
	if (mt7620_gpio_is_set(mt7620_gpio_reg_offset(GPIO_LED_POWER_SWITCH_BIT_CONTROL), mt7620_gpio_led_onoff_intp) 
		&& TP_MT7620_GPIO_FALLING_EDGE == 
		mt7620_gpio_is_set(mt7620_gpio_reg_offset(GPIO_LED_POWER_SWITCH_BIT_CONTROL), mt7620_gpio_led_onoff_edge))
	{
		if (0 == mt7620_gpio_factory_reset_inited)
		{
			return IRQ_HANDLED;
		}
		mod_timer(&mt7620_gpio_led_onoff_timer, jiffies + HZ/10);
	}
#endif    
#endif

	return IRQ_HANDLED;
}

/* ioctl for reset default detection and system led */
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,35)
long mt7620_gpio_ioctl(struct file *file, unsigned int req,
					  unsigned long arg)
#else
int mt7620_gpio_ioctl(struct inode *inode, struct file *file, 
					  unsigned int req, unsigned long arg)
#endif
{
#ifdef GPIO_SYS_LED
	u32 gpiodata;
#endif
	int* argp = (int *)arg;

	if (_IOC_TYPE(req) != TP_MT7620_GPIO_MAGIC ||
		_IOC_NR(req) < TP_MT7620_GPIO_IOCTL_BASE ||
		_IOC_NR(req) > TP_MT7620_GPIO_IOCTL_MAX)
	{
		printk("type:%d nr:%d\n", _IOC_TYPE(req), _IOC_NR(req));
		printk("mt7620_gpio_ioctl:unknown command\n");
		return -1;
	}

	switch (req)
	{
		case TP_MT7620_GPIO_BTN_READ:
			*argp = mt7620_gpio_reset_enable;
			mt7620_gpio_reset_enable = 0;
			break;
			
		case TP_MT7620_GPIO_MODE_SWITCH:
			*argp = mt7620_gpio_mode_switch_enable;
			mt7620_gpio_mode_switch_enable = 0;
			break;
				
		case TP_MT7620_GPIO_LED_WRITE:
			/*get the data of gpio*/
			#ifdef GPIO_SYS_LED
			gpiodata = mt7620_gpio_read(TP_MT7620_REG_PIODATA);
			gpiodata = mt7620_gpio_set_bit(GPIO_SYS_LED_BIT_CONTROL, (*argp), gpiodata);
			mt7620_gpio_write(TP_MT7620_REG_PIODATA, gpiodata);
			#endif
			break;
			
		default:
			printk("command not supported\n");
			return -ENOIOCTLCMD;
	}

	return 0;
}

static int mt7620_gpio_open(struct inode *inode, struct file *file)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
	MOD_INC_USE_COUNT;
#else
	try_module_get(THIS_MODULE);
#endif
	return 0;
}

static int mt7620_gpio_release(struct inode *inode, struct file *file)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
	MOD_DEC_USE_COUNT;
#else
	module_put(THIS_MODULE);
#endif
	return 0;
}

struct irqaction mt7620_gpio_irqaction = {
	.handler = mt7620_gpio_intp_gpio_irq_handler,
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,35)
	.flags = IRQF_DISABLED,
#else
	.flags = SA_INTERRUPT,
#endif
	.name = "mt7620_gpio",
};

struct file_operations mt7620_gpio_fops =
{
	owner:		THIS_MODULE,
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,35)
	unlocked_ioctl:	mt7620_gpio_ioctl,
#else
	ioctl:		mt7620_gpio_ioctl,
#endif
	open:		mt7620_gpio_open,
	release:	mt7620_gpio_release,
};

void __init mt7620_gpio_init_irq(void)
{
	setup_irq(SURFBOARDINT_GPIO, &mt7620_gpio_irqaction);
}

static int mt7620_gpio_chrdev_init(void)
{
	int cdev = 0;

	cdev = register_chrdev(TP_MT7620_DEV_MAJOR, RALINK_GPIO_DEVNAME,
			&mt7620_gpio_fops);
	
	if (cdev < 0) 
	{
		printk(KERN_ERR NAME ": unable to register character device\n");
		return cdev;
	}

	return 0;
}

#ifdef SUPPORT_HARDWARE_MULTI_MODE
#if 1
static int mt7620_gpio_init_sysMode(void)
{
	u32 gpio_data;

	gpio_data = mt7620_gpio_read(TP_MT7620_REG_PIODATA);

	//printk("\n\t\t\t>>kernelw3294<< [%s] %d,gpio_data:%x \n", __FUNCTION__, __LINE__,gpio_data);
	
	if (!mt7620_gpio_is_set(TP_MT7620_GPIODIR_BIT7, gpio_data) 
		&& mt7620_gpio_is_set(TP_MT7620_GPIODIR_BIT8, gpio_data))	// AP
	{
		g_sysMode = SYSTEM_MODE_AP;
	}
	else if (mt7620_gpio_is_set(TP_MT7620_GPIODIR_BIT7, gpio_data) 
		&& !mt7620_gpio_is_set(TP_MT7620_GPIODIR_BIT8, gpio_data) )	// APC
	{
        #ifdef MODE_APC_ROUTER
		g_sysMode = SYSTEM_MODE_APC_ROUTER;
        #endif

        #ifdef MODE_NORMAL_ROUTER
        g_sysMode = SYSTEM_MODE_NORMAL_ROUTER;
        #endif
	}
	else											// 3G
	{
		g_sysMode = SYSTEM_MODE_3G_ROUTER;    
	}
}

static int mt7620_gpio_check_sysMode(unsigned long nothing)
{
	int system_mode;
	u32 gpio_data;

	gpio_data = mt7620_gpio_read(TP_MT7620_REG_PIODATA);
	
	if (!mt7620_gpio_is_set(TP_MT7620_GPIODIR_BIT7, gpio_data) 
		&& mt7620_gpio_is_set(TP_MT7620_GPIODIR_BIT8, gpio_data))	// AP
	{
		system_mode = SYSTEM_MODE_AP;
	}
	else if (mt7620_gpio_is_set(TP_MT7620_GPIODIR_BIT7,gpio_data) 
		&& !mt7620_gpio_is_set(TP_MT7620_GPIODIR_BIT8,gpio_data) )	// APC
	{
        #ifdef MODE_APC_ROUTER
		system_mode = SYSTEM_MODE_APC_ROUTER;
        #endif

        #ifdef MODE_NORMAL_ROUTER
        system_mode = SYSTEM_MODE_NORMAL_ROUTER;
        #endif
	}
	else											// 3G
	{
		system_mode = SYSTEM_MODE_3G_ROUTER;
        
	}

	if (g_sysMode != system_mode && !g_isSoftSysModeEnable)
	{
		//reboot router
		printk("kernel, rebooting...\n");
		/* modify by xiongliuzhong, 01Apr13, as we have no hardware reset gpio */
		mt7620_gpio_mode_switch_enable = 1;
		return 0;
	}
	
	mod_timer(&mt7620_gpio_sysMode_timer, jiffies + 1 * HZ);
}

#endif

static int mt7620_gpio_sysmode_check(void)
{
	mt7620_gpio_init_sysMode();
	init_timer(&mt7620_gpio_sysMode_timer);
	mt7620_gpio_sysMode_timer.function = mt7620_gpio_check_sysMode;
	mod_timer(&mt7620_gpio_sysMode_timer, jiffies + 1 * HZ);

	return 0;
}
#endif


#ifdef DEBUG_TP_MT7620_INPUT_DATA_REGISTER
static void mt7620_gpio_input_data(void)
{
	u32 gpiodata;
	int bit7;
	int bit8;
	
	/* config gpio0-23 */
	gpiodata = mt7620_gpio_read(TP_MT7620_REG_PIODATA);
		
	/* mode switch data init */
#ifdef SUPPORT_HARDWARE_MULTI_MODE
	bit7 = ((mt7620_gpio_is_set(GPIO_MODE_SWITCH_CONTROL0, gpiodata)) ? 1:0);
	bit8 = ((mt7620_gpio_is_set(GPIO_MODE_SWITCH_CONTROL1, gpiodata)) ? 1:0);
#endif
	printk("+++++++>>>bit7: %d\n", bit7);
	printk("+++++++>>>bit8: %d\n", bit8);

	return;
}
#endif

static int __init mt7620_simple_config_init(void)
{
    
    /*init reset timer*/
	init_timer(&mt7620_gpio_rst_timer);
	mt7620_gpio_rst_timer.function = mt7620_gpio_check_rst;

#ifdef GPIO_WPS_BIT_CONTROL
	/*init wps timer*/
	init_timer(&mt7620_gpio_wps_timer);
	mt7620_gpio_wps_timer.function = mt7620_gpio_check_wps;
#endif 	

#ifdef GPIO_POWER_ONOFF_CONTROL
	init_timer(&mt7620_gpio_power_onoff_timer);
	mt7620_gpio_power_onoff_timer.function = mt7620_gpio_check_power_onoff;

	init_timer(&mt7620_gpio_power_blink_timer);
	mt7620_gpio_power_blink_timer.expires = jiffies + HZ;
	mt7620_gpio_power_blink_timer.function = mt7620_gpio_check_power_blink;
	add_timer(&mt7620_gpio_power_blink_timer);
#endif

#ifdef LED_POWER_SWITCH_ON_OFF
#if defined(CONFIG_RALINK_GPIO_RE210)
	init_timer(&mt7620_gpio_led_onoff_timer);
	mt7620_gpio_led_onoff_timer.function = mt7620_gpio_check_led_onoff;
#endif    
#endif

	/*init the gpio data*/
	mt7620_gpio_common_init();

#ifdef DEBUG_TP_MT7620_INPUT_DATA_REGISTER
	mt7620_gpio_input_data();
#endif

	/* device cdev init */
	mt7620_gpio_chrdev_init();
	
	/*create the file in proc*/
	mt7620_gpio_create_simple_config_proc();

#ifdef SUPPORT_HARDWARE_MULTI_MODE
	mt7620_gpio_sysmode_check();
#endif

#ifdef	CONFIG_TP_GPIO_EVENT
	gpio_thread_init();
#endif	/* CONFIG_TP_GPIO_EVENT */

#ifdef GPIO_POWER_SYSTEM_LED
    //gpio_thread_led_init();
#endif    

	mt7620_gpio_factory_reset_inited = 1;
	
	return 0;
}

device_initcall(mt7620_simple_config_init);
