/******************************************************************************
Copyright (c) 2009-2013 TP-Link Technologies CO.,LTD.  All rights reserved. 

File name	: br_deliver.c
Version		: v1.0, release version
Description	: This file impleaments the dual wifi repeater connection func
		  for some dual band repeaters, such as wa3500re(EU) 1.0.
 
Author		: huanglifu <huanglifu@tp-link.net>
Create date	: 2013/5/27

History		:
01, 2013/05/27 huanglifu, Created file.
02, 2013/09/23 tengfei, Modify file name and some descriptions to remove info
               related special product(wda3150).
*****************************************************************************/
#include "br_deliver.h"
#include <linux/proc_fs.h>
#include <net/arp.h>
/*global variable for free bridge fdb.*/
struct tp_db_fre_bridge_fdb_entry free_bridage_fdb;

/*create a proc dir for user layer controling.*/
struct proc_dir_entry * range_extender_config_entry = NULL;
EXPORT_SYMBOL(range_extender_config_entry);

/*a switch for enable range_extender bridge deliver function.*/
int g_range_extender_bridge_deliver_enable = 0;
static struct proc_dir_entry * range_extender_bridge_deliver_enable_entry = NULL;

/*when range_extender's 2g front is connected, set this variable value as 1.*/
int g_range_extender_2g_cli_enable = 0;
static struct proc_dir_entry *range_extender_2g_cli_enable_entry = NULL;

/*when range_extender's 5g front is connected, set this variable value as 1.*/
int g_range_extender_5g_cli_enable = 0;
static struct proc_dir_entry *range_extender_5g_cli_enable_entry = NULL;

/*a flag for dev eth/br0 using 2g or 5g link to front */
int g_range_extender_eth_to_2g_enable = 1;
static struct proc_dir_entry *range_extender_eth_to_2g_enable_entry = NULL;

/*when range_extender wireless useing 3addrees, set this variable value as 1.*/
int g_range_extender_wireless_repeater_3addrees = 1;
static struct proc_dir_entry *range_extender_wireless_repeater_3addrees_entry = NULL;

/*when range_extender dual wifi connect to same front dut, set this variable value as 1.*/
int g_range_extender_has_same_front_dut = 1;
static struct proc_dir_entry *range_extender_has_same_front_dut_entry = NULL;


/* ---!--- For Test ---!--- */
unsigned int g_range_extender_test = 1;
static struct proc_dir_entry *range_extender_test = NULL;

static void init_filter_table(struct tp_db_fre_bridge_fdb_entry *p);
static void br_do_send_arp_request_packet_proxy();

static int range_extender_bridge_deliver_enable_entry_read (char *page, char **start, off_t off,
                               int count, int *eof, void *data)
{
	GET_PROC_VAR_VALUE(g_range_extender_bridge_deliver_enable);
}

static int range_extender_bridge_deliver_enable_entry_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	SET_PROC_VAR_VALUE(g_range_extender_bridge_deliver_enable);

	range_extender_prink("%s %d range_extender_bridge_deliver_enable = %d\n", 
		__FUNCTION__, __LINE__, g_range_extender_bridge_deliver_enable);
	free_bridage_fdb.is_enabled = 0;
	return count;
}

static int range_extender_2g_cli_enable_entry_read (char *page, char **start, off_t off,
                               int count, int *eof, void *data)
{
    GET_PROC_VAR_VALUE(g_range_extender_2g_cli_enable);
}

static int range_extender_2g_cli_enable_entry_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	int old_link_state;
	
	SET_PROC_VAR_VALUE(g_range_extender_2g_cli_enable);
	old_link_state = free_bridage_fdb.link_state;
	range_extender_prink("range_extender_2g_cli_enable = %d\n", g_range_extender_2g_cli_enable);	
	free_bridage_fdb.link_state = g_range_extender_2g_cli_enable * LINK_STATE_ONLY_2G 
				  + g_range_extender_5g_cli_enable * LINK_STATE_ONLY_5G;	
	range_extender_prink("free_bridage_fdb.link_state = %d\n", free_bridage_fdb.link_state);

	if(g_range_extender_wireless_repeater_3addrees
		&& free_bridage_fdb.is_enabled 
		&& old_link_state != free_bridage_fdb.link_state)
	{
		br_do_send_arp_request_packet_proxy();
	}
	
	return count;
}

static int range_extender_5g_cli_enable_entry_read (char *page, char **start, off_t off,
                               int count, int *eof, void *data)
{
    GET_PROC_VAR_VALUE(g_range_extender_5g_cli_enable);
}

static int range_extender_5g_cli_enable_entry_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	int old_link_state;
	
	SET_PROC_VAR_VALUE(g_range_extender_5g_cli_enable);
	old_link_state = free_bridage_fdb.link_state;
	range_extender_prink("range_extender_5g_cli_enable = %d\n", g_range_extender_5g_cli_enable);
	free_bridage_fdb.link_state = g_range_extender_2g_cli_enable * LINK_STATE_ONLY_2G 
				  + g_range_extender_5g_cli_enable * LINK_STATE_ONLY_5G;	
	range_extender_prink("free_bridage_fdb.link_state = %d\n", free_bridage_fdb.link_state);

	if(g_range_extender_wireless_repeater_3addrees
		&& free_bridage_fdb.is_enabled 
		&& old_link_state != free_bridage_fdb.link_state)
	{
		br_do_send_arp_request_packet_proxy();
	}
	
	return count;
}
static int range_extender_eth_to_2g_enable_entry_read (char *page, char **start, off_t off,
                               int count, int *eof, void *data)
{
    GET_PROC_VAR_VALUE(g_range_extender_eth_to_2g_enable);
}

static int range_extender_eth_to_2g_enable_entry_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	int old_eth_goto_2g_flag;
	SET_PROC_VAR_VALUE(g_range_extender_eth_to_2g_enable);
	old_eth_goto_2g_flag = free_bridage_fdb.eth_goto_2g_flag;
	range_extender_prink("range_extender_eth_to_2g_enable = %d\n", g_range_extender_eth_to_2g_enable);
	free_bridage_fdb.eth_goto_2g_flag = g_range_extender_eth_to_2g_enable;

	if(g_range_extender_wireless_repeater_3addrees 
		&& free_bridage_fdb.is_enabled 
		&& old_eth_goto_2g_flag != free_bridage_fdb.eth_goto_2g_flag)
	{
		init_filter_table(&free_bridage_fdb);
		br_do_send_arp_request_packet_proxy();
	}
	
	return count;
}

static int range_extender_wireless_repeater_3addrees_entry_read (char *page, char **start, off_t off,
                               int count, int *eof, void *data)
{
    GET_PROC_VAR_VALUE(g_range_extender_wireless_repeater_3addrees);
}

static int range_extender_wireless_repeater_3addrees_entry_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	SET_PROC_VAR_VALUE(g_range_extender_wireless_repeater_3addrees);
	range_extender_prink("range_extender_wireless_repeater_3addrees = %d\n", g_range_extender_wireless_repeater_3addrees );
	return count;
}

static int range_extender_has_same_front_dut_entry_read (char *page, char **start, off_t off,
                               int count, int *eof, void *data)
{
	g_range_extender_has_same_front_dut = free_bridage_fdb.is_same_front_dut;
    	GET_PROC_VAR_VALUE(g_range_extender_has_same_front_dut);
}

static int range_extender_has_same_front_dut_entry_write (struct file *file, const char *buf,
                                        unsigned long count, void *data)
{
	SET_PROC_VAR_VALUE(g_range_extender_has_same_front_dut);
	free_bridage_fdb.is_same_front_dut = g_range_extender_has_same_front_dut;
	range_extender_prink("g_range_extender_has_same_front_dut = %d\n", g_range_extender_has_same_front_dut  );
	return count;
}


/*=================================*/
#if	0
static inline unsigned tf_priv(char* val)
{
	unsigned i =1, ret = 0;

	for (i = 1; i <= 6; i++)
	{
		ret |= (*(val + i) * (1 << (i -1)));
	}

	return ret;
}
#endif

static int range_extender_test_read(char *page, char **start, off_t off, int count, int *eof, void *data)
{
#if	0
	unsigned int tmp = 0;

	g_range_extender_test = tmp;
	tmp= tf_priv(free_bridage_fdb.filter_table_2g_5g[3]);
	g_range_extender_test = tmp;
	tmp = tf_priv(free_bridage_fdb.filter_table_only_2g[3]);
	g_range_extender_test |= (tmp << 0x8);
	tmp = tf_priv(free_bridage_fdb.filter_table_only_5g[3]);
	g_range_extender_test |= (tmp << 0x10);
#endif

    	GET_PROC_VAR_VALUE(g_range_extender_test);
}

static int range_extender_test_write(struct file *file, const char *buf, unsigned long count, void *data)
{
	SET_PROC_VAR_VALUE(g_range_extender_test);
}
/*=================================*/

int create_range_extender_config_proc()
{
	if (range_extender_config_entry != NULL)
	{
		printk ("Already have a proc entry for /proc/range_extender_config!\n");
		return -ENOENT;
	}

	range_extender_config_entry = proc_mkdir("range_extender_config", NULL);
	if (!range_extender_config_entry)
	{
		return -ENOENT;
	}

	range_extender_bridge_deliver_enable_entry = create_proc_entry("range_extender_bridge_deliver_enable", 0666, range_extender_config_entry);
	if (!range_extender_bridge_deliver_enable_entry)
	{
		return -ENOENT;
	}
	range_extender_bridge_deliver_enable_entry->read_proc = range_extender_bridge_deliver_enable_entry_read;
	range_extender_bridge_deliver_enable_entry->write_proc = range_extender_bridge_deliver_enable_entry_write; 


	range_extender_2g_cli_enable_entry = create_proc_entry("range_extender_2g_cli_enable", 0666, range_extender_config_entry);
	if (!range_extender_2g_cli_enable_entry)
	{
		return -ENOENT;
	}
	range_extender_2g_cli_enable_entry->read_proc = range_extender_2g_cli_enable_entry_read;
	range_extender_2g_cli_enable_entry->write_proc = range_extender_2g_cli_enable_entry_write; 

	range_extender_5g_cli_enable_entry = create_proc_entry("range_extender_5g_cli_enable", 0666, range_extender_config_entry);
	if (!range_extender_5g_cli_enable_entry)
	{
		return -ENOENT;

	}
	range_extender_5g_cli_enable_entry->read_proc = range_extender_5g_cli_enable_entry_read;
	range_extender_5g_cli_enable_entry->write_proc = range_extender_5g_cli_enable_entry_write; 

	range_extender_eth_to_2g_enable_entry = create_proc_entry("range_extender_eth_to_2g_enable", 0666, range_extender_config_entry);
	if (!range_extender_eth_to_2g_enable_entry)
	{
		return -ENOENT;
	}

	range_extender_eth_to_2g_enable_entry->read_proc = range_extender_eth_to_2g_enable_entry_read;
	range_extender_eth_to_2g_enable_entry->write_proc = range_extender_eth_to_2g_enable_entry_write; 

	range_extender_wireless_repeater_3addrees_entry = create_proc_entry("range_extender_repeater_is_3addrees", 0666, range_extender_config_entry);
	if (!range_extender_wireless_repeater_3addrees_entry)
	{
		return -ENOENT;
	}
	range_extender_wireless_repeater_3addrees_entry->read_proc = range_extender_wireless_repeater_3addrees_entry_read;
	range_extender_wireless_repeater_3addrees_entry->write_proc = range_extender_wireless_repeater_3addrees_entry_write; 

	range_extender_has_same_front_dut_entry = create_proc_entry("range_extender_front_dut_num", 0666, range_extender_config_entry);
	if (!range_extender_has_same_front_dut_entry)
	{
		return -ENOENT;
	}
	range_extender_has_same_front_dut_entry->read_proc = range_extender_has_same_front_dut_entry_read;
	range_extender_has_same_front_dut_entry->write_proc = range_extender_has_same_front_dut_entry_write; 



	/* For Test */
	range_extender_test = create_proc_entry("test", 0666, range_extender_config_entry);
	if (!range_extender_test)
	{
		return -ENOENT;
	}
	range_extender_test->read_proc = range_extender_test_read;
	range_extender_test->write_proc = range_extender_test_write; 
	/* For Test */

}

int init_free_bridge_fdb(void)
{

	struct 	net_device* tmpDev = NULL;
	struct tp_db_fre_bridge_fdb_entry *p = &free_bridage_fdb;
	struct net_bridge_port* tmp_dev_br_port = NULL;
	unsigned int br_sub_dev_index_count = 0;
	
	if(LINK_STATE_ON != g_range_extender_bridge_deliver_enable)
	{
		goto INIT_FAIL;
	}
	
	memset(p, 0, sizeof(struct tp_db_fre_bridge_fdb_entry));	
	
	br_sub_dev_index_count = 0;
	/*back of dual repeater*/
	GET_DEV_INDEX_BY_NAME(tmpDev, p->ap_dev_2g_idx, BRIDGE_DELIVER_2G_DEV_NAME);
	GET_DEV_INDEX_BY_NAME(tmpDev, p->ap_dev_5g_idx, BRIDGE_DELIVER_5G_DEV_NAME);	
	GET_DEV_INDEX_BY_NAME(tmpDev, p->eth_dev_idx, BRIDGE_DELIVER_LAN_ETH_NAME);	
	GET_DEV_INDEX_BY_NAME(tmpDev, p->br0_dev_idx, BRIDGE_DELIVER_NAME);	
	/*front of dual repeater*/
	GET_DEV_INDEX_BY_NAME(tmpDev, p->client_dev_2g_idx, BRIDGE_DELIVER_2G_STA_NAME);
	GET_DEV_INDEX_BY_NAME(tmpDev, p->client_dev_5g_idx, BRIDGE_DELIVER_5G_STA_NAME);
	
	GET_DEV_BY_NAME(p->client_2g_dev, BRIDGE_DELIVER_2G_STA_NAME);
	GET_DEV_BY_NAME(p->client_5g_dev, BRIDGE_DELIVER_5G_STA_NAME);	
	GET_DEV_BY_NAME(p->br0_dev, BRIDGE_DELIVER_NAME);	
	
	tmp_dev_br_port = br_port_get_rcu(p->client_2g_dev);
	if (tmp_dev_br_port == NULL)
	{
		goto INIT_FAIL;
	}
	p->br0_bri_dev = tmp_dev_br_port->br;
	
	p->is_enabled = LINK_STATE_ON;
	p->link_state = g_range_extender_2g_cli_enable * LINK_STATE_ONLY_2G 
				  + g_range_extender_5g_cli_enable * LINK_STATE_ONLY_5G;	
	p->eth_goto_2g_flag = g_range_extender_eth_to_2g_enable;	
	/* set its value as default value. */
	p->is_same_front_dut = g_range_extender_has_same_front_dut;

	/*
	**	Put init_filter_table at last to fix the problem that traffic from eth will go through apclii0 link
	@@	By TengFei.
	*/
	init_filter_table(&free_bridage_fdb);
	
	return LINK_STATE_ON;
	
INIT_FAIL:	
	p->is_enabled = LINK_STATE_OFF;
	g_range_extender_bridge_deliver_enable = 0;
	return LINK_STATE_OFF;
}

static void init_filter_table(struct tp_db_fre_bridge_fdb_entry *p)
{
	if (NULL == p)
	{	
		return;
	}
	
	memset(&p->filter_table_2g_5g, 0, sizeof(p->filter_table_2g_5g));
	memset(&p->filter_table_only_2g, 0, sizeof(p->filter_table_only_2g));
	memset(&p->filter_table_only_5g, 0, sizeof(p->filter_table_only_5g));
	
/*
 * FILETER RULE TABLE FOR 2G & 5G.
 * when value is 1, it mean that from dev should not deliver to dev. else is allowed.
 * from\to| AP2G | AP5G | CLI2G | CLI5G | BR0 | ETH 
 * AP2G   |  -   |  0   |   0   |   1   |  0  |  0
 * AP5G   |  0   |  -   |   1   |   0   |  0  |  0
 * CLI2G  |  0   |  1   |   -   |   1   |  0  |  0
 * CLI5G  |  1   |  0   |   1   |   -   |  1  |  1
 * BR0    |  0   |  0   |   0   |   1   |  -  |  0
 * ETH    |  0   |  0   |   0   |   1   |  0  |  -
*/
	FILTER_SET_VALUE_ON(p->filter_table_2g_5g, p->client_dev_2g_idx, p->client_dev_5g_idx);
	FILTER_SET_VALUE_ON(p->filter_table_2g_5g, p->client_dev_2g_idx, p->ap_dev_5g_idx);
	FILTER_SET_VALUE_ON(p->filter_table_2g_5g, p->ap_dev_5g_idx, p->client_dev_2g_idx);
	FILTER_SET_VALUE_ON(p->filter_table_2g_5g, p->ap_dev_2g_idx, p->client_dev_5g_idx);
	FILTER_SET_VALUE_ON(p->filter_table_2g_5g, p->client_dev_5g_idx, p->client_dev_2g_idx);
	FILTER_SET_VALUE_ON(p->filter_table_2g_5g, p->client_dev_5g_idx, p->ap_dev_2g_idx);	
	FILTER_SET_VALUE_ON(p->filter_table_2g_5g, p->br0_dev_idx, 
		GET_2G_5G_DEV_INDEX_BY_FLAG(p->eth_goto_2g_flag, p->client_dev_5g_idx, p->client_dev_2g_idx));	
	FILTER_SET_VALUE_ON(p->filter_table_2g_5g, 
		GET_2G_5G_DEV_INDEX_BY_FLAG(p->eth_goto_2g_flag, p->client_dev_5g_idx, p->client_dev_2g_idx), 
		p->br0_dev_idx);	
	FILTER_SET_VALUE_ON(p->filter_table_2g_5g, p->eth_dev_idx, 
		GET_2G_5G_DEV_INDEX_BY_FLAG(p->eth_goto_2g_flag, p->client_dev_5g_idx, p->client_dev_2g_idx));
	FILTER_SET_VALUE_ON(p->filter_table_2g_5g, 
		GET_2G_5G_DEV_INDEX_BY_FLAG(p->eth_goto_2g_flag, p->client_dev_5g_idx, p->client_dev_2g_idx), 
		p->eth_dev_idx);

/*
 * FILETER TABLE FOR 2G ONLY.
 * when value is 1, it mean that from dev should not deliver to dev. else is allowed.
 * from\to| AP2G | AP5G | CLI2G | CLI5G | BR0 | ETH 
 * AP2G   |  -   |  0   |   0   |   1   |  0  |  0
 * AP5G   |  0   |  -   |   0   |   1   |  0  |  0
 * CLI2G  |  0   |  0   |   -   |   1   |  0  |  0
 * CLI5G  |  1   |  1   |   1   |   -   |  1  |  1
 * BR0    |  0   |  0   |   0   |   1   |  -  |  0
 * ETH    |  0   |  0   |   0   |   1   |  0  |  -
*/
	FILTER_SET_VALUE_ON(p->filter_table_only_2g, p->client_dev_5g_idx, p->client_dev_2g_idx);
	FILTER_SET_VALUE_ON(p->filter_table_only_2g, p->client_dev_5g_idx, p->ap_dev_2g_idx);
	FILTER_SET_VALUE_ON(p->filter_table_only_2g, p->client_dev_5g_idx, p->ap_dev_5g_idx);	
	FILTER_SET_VALUE_ON(p->filter_table_only_2g, p->client_dev_5g_idx, p->br0_dev_idx);
	FILTER_SET_VALUE_ON(p->filter_table_only_2g, p->client_dev_5g_idx, p->eth_dev_idx);
	
	FILTER_SET_VALUE_ON(p->filter_table_only_2g, p->ap_dev_5g_idx, p->client_dev_5g_idx);
	FILTER_SET_VALUE_ON(p->filter_table_only_2g, p->client_dev_2g_idx, p->client_dev_5g_idx);
	FILTER_SET_VALUE_ON(p->filter_table_only_2g, p->ap_dev_2g_idx, p->client_dev_5g_idx);
	FILTER_SET_VALUE_ON(p->filter_table_only_2g, p->eth_dev_idx, p->client_dev_5g_idx);
	FILTER_SET_VALUE_ON(p->filter_table_only_2g, p->br0_dev_idx, p->client_dev_5g_idx);	
/*
 * FILETER TABLE FOR 5G ONLY.
 * when value is 1, it mean that from dev should not deliver to dev. else is allowed.
 * from\to| AP2G | AP5G | CLI2G | CLI5G | BR0 | ETH 
 * AP2G   |  -   |  0   |   1   |   0   |  0  |  0
 * AP5G   |  0   |  -   |   1   |   0   |  0  |  0
 * CLI2G  |  1   |  1   |   -   |   1   |  1  |  1
 * CLI5G  |  0   |  0   |   1   |   -   |  0  |  0
 * BR0    |  0   |  0   |   1   |   0   |  -  |  0
 * ETH    |  0   |  0   |   1   |   0   |  0  |  -
*/

	FILTER_SET_VALUE_ON(p->filter_table_only_5g, p->client_dev_2g_idx, p->client_dev_5g_idx);
	FILTER_SET_VALUE_ON(p->filter_table_only_5g, p->client_dev_2g_idx, p->ap_dev_2g_idx);
	FILTER_SET_VALUE_ON(p->filter_table_only_5g, p->client_dev_2g_idx, p->ap_dev_5g_idx);
	FILTER_SET_VALUE_ON(p->filter_table_only_5g, p->client_dev_2g_idx, p->br0_dev_idx);
	FILTER_SET_VALUE_ON(p->filter_table_only_5g, p->client_dev_2g_idx, p->eth_dev_idx);
	
	FILTER_SET_VALUE_ON(p->filter_table_only_5g, p->ap_dev_5g_idx, p->client_dev_2g_idx);
	FILTER_SET_VALUE_ON(p->filter_table_only_5g, p->client_dev_5g_idx, p->client_dev_2g_idx);
	FILTER_SET_VALUE_ON(p->filter_table_only_5g, p->ap_dev_2g_idx, p->client_dev_2g_idx);
	FILTER_SET_VALUE_ON(p->filter_table_only_5g, p->eth_dev_idx, p->client_dev_2g_idx);
	FILTER_SET_VALUE_ON(p->filter_table_only_5g, p->br0_dev_idx, p->client_dev_2g_idx);
	
	return;
}

static unsigned tp_get_br_front_port_ip(unsigned int src_ip)
{
	struct tp_db_fre_bridge_fdb_entry *p = &free_bridage_fdb;
 	struct net_bridge *br = p->br0_bri_dev;
	unsigned int dst_ip = 0;	
	int i;

	spin_lock_bh(&br->hash_lock);
	for (i = 0; i < BR_HASH_SIZE && !dst_ip; i++) {
		struct net_bridge_fdb_entry *f;
		struct hlist_node *h, *n;

		hlist_for_each_entry_safe(f, h, n, &br->hash[i], hlist) {			
		 	
			if (f->dst->dev->br_port_index == p->client_dev_2g_idx
				|| f->dst->dev->br_port_index == p->client_dev_5g_idx)
			{				
				if(src_ip != f->ip){
					dst_ip = f->ip;
					break;
				}
			}
		}
	}
	spin_unlock_bh(&br->hash_lock);	
	
	return dst_ip;	
}

static void br_do_br0_send_arp_request_packet_proxy()
{
	struct tp_db_fre_bridge_fdb_entry *p = &free_bridage_fdb;
	struct net_device* tmp_dev = NULL;
	struct in_device *pdev_ipaddr=NULL;
	unsigned int dst_ip = 0;
	unsigned int src_ip = 0;
	
	if (!p->br0_dev || !p->br0_dev->ip_ptr)
	{
		return;
	}
	pdev_ipaddr = (struct in_device *)p->br0_dev->ip_ptr;
	if(!pdev_ipaddr || !pdev_ipaddr->ifa_list)
	{
		return;
	}
	src_ip = pdev_ipaddr->ifa_list->ifa_local;
	
	if (!src_ip)
	{
		return;
	}

	switch (p->link_state)
	{
		case LINK_STATE_ALL_2G_5G:
			tmp_dev = (p->eth_goto_2g_flag) ? p->client_2g_dev : p->client_5g_dev;
		break;
		
		case LINK_STATE_ONLY_2G:
			tmp_dev = p->client_2g_dev;
		break;
		
		case LINK_STATE_ONLY_5G:
			tmp_dev = p->client_5g_dev;
		break;		
	}
	
	dst_ip = tp_get_br_front_port_ip(src_ip);
	/*
	**	Sometimes tmp_dev(client_2g_dev or client_5g_dev) would be free, and this may cause kernel panic.
	**	So it is necessary to check existence of "tmp_dev".
	@@	TengFei, 13/11/11
	*/
	if (tmp_dev && dst_ip)
	{ 
		arp_send(ARPOP_REQUEST, ETH_P_ARP, dst_ip, tmp_dev, src_ip, NULL, p->br0_dev->dev_addr, NULL);
	}
}

static void br_do_send_arp_request_packet_proxy()
{
	struct tp_db_fre_bridge_fdb_entry *p = &free_bridage_fdb;
 	struct net_bridge *br = p->br0_bri_dev;
	struct net_device* tmp_dev = NULL;
	unsigned int dst_ip = 0;
	int i;

	br_do_br0_send_arp_request_packet_proxy();
	
	spin_lock_bh(&br->hash_lock);
	for (i = 0; i < BR_HASH_SIZE; i++) {
		struct net_bridge_fdb_entry *f;
		struct hlist_node *h, *n;

		hlist_for_each_entry_safe(f, h, n, &br->hash[i], hlist) {
			if (f->dst->dev->br_port_index == p->client_dev_2g_idx
				|| f->dst->dev->br_port_index == p->client_dev_5g_idx
				|| !f->ip)
			{
				continue;
			}		
			switch (p->link_state)
			{
				case LINK_STATE_ALL_2G_5G:
					if (f->dst->dev->br_port_index == p->eth_dev_idx
						|| f->dst->dev->br_port_index == p->br0_dev_idx)
					{
						tmp_dev = (p->eth_goto_2g_flag) ? p->client_2g_dev : p->client_5g_dev;
					}else if (f->dst->dev->br_port_index == p->ap_dev_2g_idx)
					{
						tmp_dev = p->client_2g_dev;
					}else if (f->dst->dev->br_port_index == p->ap_dev_5g_idx)
					{
						tmp_dev = p->client_5g_dev;
					}
					break;
				
				case LINK_STATE_ONLY_2G:
					tmp_dev = p->client_2g_dev;
					break;
				
				case LINK_STATE_ONLY_5G:
					tmp_dev = p->client_5g_dev;
					break;			
			}
			
			dst_ip = 0;
			dst_ip = tp_get_br_front_port_ip(f->ip);
			/*
			**	Sometimes tmp_dev(client_2g_dev or client_5g_dev) would be free, and this may cause kernel panic.
			**	So it is necerrary to check existence of "tmp_dev".
			@@	TengFei, 13/11/11
			*/
			if (tmp_dev && dst_ip)
			{
				arp_send(ARPOP_REQUEST,ETH_P_ARP, dst_ip, tmp_dev, f->ip, NULL, f->addr.addr, NULL);
			}
			
		}
	}
	spin_unlock_bh(&br->hash_lock);
}


int br_should_deliver_packet(struct sk_buff *skb)
{
	const unsigned char *mac_src = NULL;
	struct tp_db_fre_bridge_fdb_entry *p = &free_bridage_fdb;
	int is_cli_2g_5g_mac = 0;
	
	CHECK_AND_INIT_TABLE(1);
	
	switch (p->link_state)
	{
		case LINK_STATE_ALL_2G_5G:
		case LINK_STATE_NONE:
			if (skb->dev->br_port_index == p->client_dev_5g_idx
				|| skb->dev->br_port_index == p->client_dev_2g_idx)
			{
				return br_fdb_check_ring_packet(p->br0_bri_dev, eth_hdr(skb)->h_source, p);
			}
			break;
		
		case LINK_STATE_ONLY_2G:
			if ( skb->dev->br_port_index == p->client_dev_5g_idx)
			{
				return 0;
			}
			break;
		
		case LINK_STATE_ONLY_5G:
			if (skb->dev->br_port_index == p->client_dev_2g_idx)
			{
				return 0;
			}
			break;
	}			
	return 1;
}
int br_db_fre_should_deliver(struct net_bridge_port *prev, struct sk_buff *skb)
{	
	struct net_device *psrc_dev = skb->dev;
	struct net_device *pdst_dev = prev->dev;
	struct tp_db_fre_bridge_fdb_entry *p = &free_bridage_fdb;
	int result = 0;	

	if (psrc_dev == NULL || pdst_dev == NULL)
	{
		return 0;
	}
	
	CHECK_AND_INIT_TABLE(0);

	if (DUAL_REPEATER_HAS_SAME_FROMT_DUT != p->is_same_front_dut)
	{
		return 0;
	}
	switch (p->link_state)
	{
		case LINK_STATE_ALL_2G_5G:
			if (IS_DEV_FILTERED(p->filter_table_2g_5g, psrc_dev, pdst_dev))
			{
				result = 1;
			}
			break;
		
		case LINK_STATE_ONLY_2G:
			if (IS_DEV_FILTERED(p->filter_table_only_2g, psrc_dev, pdst_dev))
			{
				result = 1;
			}
			break;
		
		case LINK_STATE_ONLY_5G:
			if (IS_DEV_FILTERED(p->filter_table_only_5g, psrc_dev, pdst_dev))
			{
				result = 1;
			}
			break;
	}
	
	return result;
}

int br_db_fre_fdb_lookup(struct net_bridge_port **to, struct sk_buff *skb)
{	

	struct net_device *psrc_dev = skb->dev;
	struct net_device *pdst_dev = (*to)->dev;
	struct tp_db_fre_bridge_fdb_entry *p = &free_bridage_fdb;
	struct net_device* tmp_dev = NULL;
	struct net_bridge_port* tmp_dev_br_port = NULL;

	if (psrc_dev == NULL || pdst_dev == NULL)
	{
		return 0;
	}

	CHECK_AND_INIT_TABLE(0);

	if (DUAL_REPEATER_HAS_SAME_FROMT_DUT != p->is_same_front_dut)
	{
		return 0;
	}	

	tmp_dev = NULL;
	switch(p->link_state)
	{
		case LINK_STATE_ALL_2G_5G:						
			if (IS_DEV_FILTERED(p->filter_table_2g_5g, psrc_dev, pdst_dev)
				&&(pdst_dev->br_port_index == p->client_dev_2g_idx 
				|| pdst_dev->br_port_index == p->client_dev_5g_idx))
			{
				tmp_dev = (pdst_dev->br_port_index == p->client_dev_2g_idx) ? p->client_5g_dev : p->client_2g_dev;				
			}
		break;

		case LINK_STATE_ONLY_2G:
			 if (pdst_dev->br_port_index == p->client_dev_5g_idx)
			 {
			 	tmp_dev = p->client_2g_dev;
			 }
			break;

		case LINK_STATE_ONLY_5G:
			if (pdst_dev->br_port_index == p->client_dev_2g_idx)
			 {
			 	tmp_dev = p->client_5g_dev;
			 }
			break;
	}

	if (tmp_dev && (tmp_dev_br_port = br_port_get_rcu(tmp_dev)))
	{
		*to = tmp_dev_br_port;
	}
	return 0;
}


/*
**	For MTK platform, we should update mapped mac by apcli0/apclii0 for ra0/rai0/eth2.1.
@@	TengFei, 14/04/10.
*/
unsigned br_fdb_update_mapped_hwaddr(struct net_bridge *br, struct net_bridge_port *source,
		   struct sk_buff *skb)
{
	struct tp_db_fre_bridge_fdb_entry *p = &free_bridage_fdb;
	struct net_device *tmp_dev = skb->dev;
	unsigned char mapped_mac[ETH_ALEN];

	CHECK_AND_INIT_TABLE(0);

	if (p->link_state == LINK_STATE_ALL_2G_5G)
	{
		if (tmp_dev)
		{	
			if (tmp_dev->br_port_index == p->ap_dev_2g_idx ||
				tmp_dev->br_port_index == p->ap_dev_5g_idx ||
				tmp_dev->br_port_index == p->eth_dev_idx)
			{
				if (p->client_5g_dev == NULL)
				{
					return 0;
				}

				memcpy(mapped_mac, p->client_5g_dev->dev_addr, ETH_ALEN/2);
				memcpy(mapped_mac + ETH_ALEN/2, eth_hdr(skb)->h_source + ETH_ALEN/2, ETH_ALEN/2);
				/* MAC Repeater has been improved */
				mapped_mac[0] |= 0x02;
				
				br_fdb_update(br, source, mapped_mac);
			}
		}
	}

	return 0;
}

